#!/usr/local/env python
#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Partial tests for the DiracILC module."""

from __future__ import absolute_import
import sys
import unittest
from mock import patch, MagicMock as Mock

from DIRAC import S_OK, S_ERROR
from Tests.Utilities.GeneralUtils import assertEqualsImproved, assertDiracFailsWith, \
    assertDiracSucceeds, assertDiracSucceedsWith, assertDiracFailsWith_equals, assertMockCalls

__RCSID__ = "$Id$"

MODULE_NAME = 'ILCDIRAC.Interfaces.API.DiracILC'

# pylint: disable=protected-access,too-many-public-methods


class DiracILCTestCase(unittest.TestCase):
  """Base class for the DiracILC test cases."""

  def setUp(self):
    """set up the objects."""
    ops_mock = Mock()
    mocked_modules = {'DIRAC.ConfigurationSystem.Client.Helpers.Operations': ops_mock}
    self.module_patcher = patch.dict(sys.modules, mocked_modules)
    self.module_patcher.start()
    from ILCDIRAC.Interfaces.API.DiracILC import DiracILC
    self.dilc = DiracILC()

    def setOptions(*args):
      if 'SingleReplicaSEs' in args[0]:
        return ['SE']
      if 'Minimum' in args[0]:
        return 1
      if args[0].endswith('PreferredSEs'):
        return ['Awesome-Tape-SE']

    ops_mock = Mock()
    ops_mock.getValue = Mock()
    ops_mock.getValue.side_effect = setOptions
    self.dilc.ops = ops_mock

  def tearDown(self):
    self.module_patcher.stop()

  def test_getprocesslist(self):
    with patch('%s.gConfig.getValue' % MODULE_NAME, new=Mock(return_value='some_gconf_testval')) as conf_mock, \
        patch('%s.ProcessList' % MODULE_NAME, new=Mock()) as pl_mock:
      res = self.dilc.getProcessList()
      pl_mock.assert_called_once_with('some_gconf_testval')
      assertEqualsImproved(res, pl_mock(), self)
      conf_mock.assert_called_once_with('/LocalSite/ProcessListPath', '')

  def test_getprocesslist_nopath(self):
    ops_mock = Mock()
    ops_mock.getValue.return_value = ''
    self.dilc.ops = ops_mock
    with patch('%s.gConfig.getValue' % MODULE_NAME, new=Mock(return_value='')) as conf_mock, \
         patch('%s.ProcessList' % MODULE_NAME, new=Mock()) as pl_mock:
      res = self.dilc.getProcessList()
      pl_mock.assert_called_once_with('')
      assertEqualsImproved(res, pl_mock(), self)
      conf_mock.assert_called_once_with('/LocalSite/ProcessListPath', '')
      ops_mock.getValue.assert_called_once_with('/ProcessList/Location', '')

  def test_presubmissionchecks_notoktosubmit(self):
    job_mock = Mock()
    job_mock.oktosubmit = False
    assertDiracFailsWith(self.dilc.preSubmissionChecks(job_mock, None),
                          'you should use job.submit(dirac)', self)

  def test_presubmissionchecks_checkfails(self):
    job_mock = Mock()
    job_mock.oktosubmit = True
    with patch('%s.DiracILC._do_check' % MODULE_NAME, new=Mock(return_value=S_ERROR('mytest_check_failed'))):
      assertDiracFailsWith(self.dilc.preSubmissionChecks(job_mock, None),
                            'mytest_check_failed', self)

  def test_presubmissionchecks_askuser_fails(self):
    job_mock = Mock()
    job_mock.oktosubmit = True
    job_mock._askUser.return_value = S_ERROR('user says no')
    self.dilc.checked = False
    with patch('%s.DiracILC._do_check' % MODULE_NAME, new=Mock(return_value=S_OK())):
      assertDiracFailsWith(self.dilc.preSubmissionChecks(job_mock, None),
                            'user says no', self)

  def test_checkparams(self):
    job_mock = Mock()
    job_mock.errorDict = {}
    with patch('%s.DiracILC.preSubmissionChecks' % MODULE_NAME, new=Mock(return_value=S_OK('mytest'))) as check_mock:
      assertDiracSucceedsWith(self.dilc.checkparams(job_mock), 'mytest', self)
      check_mock.assert_called_once_with(job_mock, mode='')

  def test_checkparams_fails(self):
    job_mock = Mock()
    job_mock.errorDict = {'myerror1': ['Terrible failure'], 'last_error': ['True']}
    assertDiracFailsWith_equals(self.dilc.checkparams(job_mock),
                                 {'myerror1': ['Terrible failure'], 'last_error': ['True']}, self)

  def test_giveprocesslist(self):
    self.dilc.processList = '13985u185r9135r'
    assertEqualsImproved(self.dilc.giveProcessList(), '13985u185r9135r', self)

  def test_giveprocesslist_empty(self):
    self.dilc.processList = ''
    assertEqualsImproved(self.dilc.giveProcessList(), '', self)

  def test_giveprocesslist_false(self):
    self.dilc.processList = False
    assertEqualsImproved(self.dilc.giveProcessList(), False, self)

  def test_giveprocesslist_none(self):
    self.dilc.processList = None
    assertEqualsImproved(self.dilc.giveProcessList(), None, self)

  def test_retrievelfns_norepo(self):
    self.dilc.jobRepo = None
    assertDiracSucceeds(self.dilc.retrieveRepositoryOutputDataLFNs(), self)

  def test_retrievelfns(self):
    repo_mock = Mock()
    ret_dict = {'1': {'State': 'Done', 'UserOutputData': '1389'}, '2': {},
                 '3': {'State': 'secret_teststate'}, '4': {'State': 'invalid_state'},
                 '5': {'State': 'Done', 'UserOutputData': 0}, '6': {'ignore_me': True},
                 '7': {'State': 'secret_teststate', 'UserOutputData': 0},
                 '148': {'State': 'Done', 1: False, True: 941, 'values_': 'keys'}}
    repo_mock.readRepository.return_value = S_OK(ret_dict)
    self.dilc.jobRepo = repo_mock
    with patch('%s.DiracILC.getJobParameters' % MODULE_NAME,
               new=Mock(side_effect=[S_OK({'UploadedOutputData': '/my/test/lfn1'}),
                                     S_ERROR(),
                                     S_OK({}),
                                     S_OK({'some_entries': 'some_values',
                                           1: True,
                                           'UploadedOutputData': '/more_lfns/append/testlfn.log'})])) as param_mock:
      assertEqualsImproved(self.dilc.retrieveRepositoryOutputDataLFNs(['Done', 'secret_teststate']),
                            ['/my/test/lfn1', '/more_lfns/append/testlfn.log'], self)
      assertMockCalls(param_mock, [3, 5, 7, 148], self)

  def test_docheck_checksandbox_fails(self):
    job_mock = Mock()
    job_mock.inputsandbox = ['mysandbox', 'other_value']
    with patch('%s.DiracILC.checkInputSandboxLFNs' % MODULE_NAME, new=Mock(return_value=S_ERROR('test_err_sandbox'))) as check_mock:
      assertDiracFailsWith(self.dilc._do_check(job_mock), 'test_err_sandbox', self)
      check_mock.assert_called_once_with(job_mock)

  def test_docheck_too_many_lists(self):
    job_mock = Mock()
    job_mock.inputsandbox = ['mysandbox', ['im_a_sandbox_file.stdhep', ['evil_list', 'deletethis']]]
    assertDiracFailsWith(self.dilc._do_check(job_mock), 'too many lists of lists in the input sandbox', self)

  def test_docheck_checkapps_fails(self):
    platform_mock = Mock()
    platform_mock.getValue.return_value = 'pf14081'
    apps_mock = Mock()
    apps_mock.getValue.return_value = 'Myapp1v.9.2.1;other_Appv.91.3;more_Dependencies.1;LasT_APP.0'
    param_dict = {'Platform': platform_mock, 'SoftwarePackages': apps_mock}
    job_mock = Mock()
    wf_mock = Mock()
    wf_mock.findParameter.side_effect = lambda param_name: param_dict[param_name]
    job_mock.inputsandbox = ['mysandbox', 'other_value', ['sandbox_file1.txt', 'sandbox_file2.log',
                                                            'last.file']]
    job_mock._resolveInputSandbox.return_value = ['resolved_file.1.txt', 'other_resolved_file.txt']
    job_mock.workflow = wf_mock
    with patch('%s.DiracILC.checkInputSandboxLFNs' % MODULE_NAME, new=Mock(return_value=S_OK())) as checksb_mock, \
         patch('%s.DiracILC._checkapp' % MODULE_NAME, new=Mock(side_effect=[S_OK()] * 3 + [S_ERROR('checkapp_failed_testme')])) as checkapp_mock:
      assertDiracFailsWith(self.dilc._do_check(job_mock), 'checkapp_failed_testme', self)
      checksb_mock.assert_called_once_with(job_mock)
      job_mock._resolveInputSandbox.assert_called_once_with(['mysandbox', 'other_value',
                                                               'sandbox_file1.txt', 'sandbox_file2.log',
                                                               'last.file'])
      job_mock._addParameter.assert_called_once_with(
          wf_mock, 'InputSandbox', 'JDL', 'resolved_file.1.txt;other_resolved_file.txt', 'Input sandbox file list')
      assertMockCalls(checkapp_mock, [('pf14081', 'myapp1v', '9.2.1'),
                                        ('pf14081', 'other_appv', '91.3'),
                                        ('pf14081', 'more_dependencies', '1'),
                                        ('pf14081', 'last_app', '0')], self)

  def test_docheck_checkoutputpath_fails(self):
    platform_mock = Mock()
    platform_mock.getValue.return_value = 'pf14081'
    apps_mock = Mock()
    apps_mock.getValue.return_value = 'Myapp1v.9.2.1;other_Appv.91.3;more_Dependencies.1;LasT_APP.0'
    path_mock = Mock()
    path_mock.getValue.return_value = 'path1948512895'
    param_dict = {'Platform': platform_mock, 'SoftwarePackages': apps_mock, 'UserOutputPath': path_mock}
    job_mock = Mock()
    wf_mock = Mock()
    wf_mock.findParameter.side_effect = lambda param_name: param_dict[param_name]
    job_mock.inputsandbox = ['mysandbox', 'other_value', ['sandbox_file1.txt', 'sandbox_file2.log',
                                                            'last.file']]
    job_mock._resolveInputSandbox.return_value = ['resolved_file.1.txt', 'other_resolved_file.txt']
    job_mock.workflow = wf_mock
    with patch('%s.DiracILC.checkInputSandboxLFNs' % MODULE_NAME, new=Mock(return_value=S_OK())) as checksb_mock, \
         patch('%s.DiracILC._checkapp' % MODULE_NAME, new=Mock(return_value=S_OK())) as checkapp_mock, \
         patch('%s.DiracILC._checkoutputpath' % MODULE_NAME, new=Mock(return_value=S_ERROR('outputpath_check_testerr'))) as checkpath_mock:
      assertDiracFailsWith(self.dilc._do_check(job_mock), 'outputpath_check_testerr', self)
      checksb_mock.assert_called_once_with(job_mock)
      job_mock._resolveInputSandbox.assert_called_once_with(['mysandbox', 'other_value',
                                                               'sandbox_file1.txt', 'sandbox_file2.log',
                                                               'last.file'])
      job_mock._addParameter.assert_called_once_with(
          wf_mock, 'InputSandbox', 'JDL', 'resolved_file.1.txt;other_resolved_file.txt', 'Input sandbox file list')
      assertMockCalls(checkapp_mock, [('pf14081', 'myapp1v', '9.2.1'),
                                        ('pf14081', 'other_appv', '91.3'),
                                        ('pf14081', 'more_dependencies', '1'),
                                        ('pf14081', 'last_app', '0')], self)
      checkpath_mock.assert_called_once_with('path1948512895')

  def test_docheck_checkconsistency_fails(self):
    platform_mock = Mock()
    platform_mock.getValue.return_value = 'pf14081'
    apps_mock = Mock()
    apps_mock.getValue.return_value = 'Myapp1v.9.2.1;other_Appv.91.3;more_Dependencies.1;LasT_APP.0'
    path_mock = Mock()
    path_mock.getValue.return_value = 'path1948512895'
    data_mock = Mock()
    data_mock.getValue.return_value = 'data1389518'
    param_dict = {'Platform': platform_mock, 'SoftwarePackages': apps_mock,
                   'UserOutputPath': path_mock, 'UserOutputData': data_mock}
    job_mock = Mock()
    job_mock.addToOutputSandbox = 'job_sandbox13895'
    wf_mock = Mock()
    wf_mock.findParameter.side_effect = lambda param_name: param_dict[param_name]
    job_mock.inputsandbox = ['mysandbox', 'other_value', ['sandbox_file1.txt', 'sandbox_file2.log',
                                                            'last.file']]
    job_mock._resolveInputSandbox.return_value = ['resolved_file.1.txt', 'other_resolved_file.txt']
    job_mock.workflow = wf_mock
    with patch('%s.DiracILC.checkInputSandboxLFNs' % MODULE_NAME, new=Mock(return_value=S_OK())) as checksb_mock, \
         patch('%s.DiracILC._checkapp' % MODULE_NAME, new=Mock(return_value=S_OK())) as checkapp_mock, \
         patch('%s.DiracILC._checkoutputpath' % MODULE_NAME, new=Mock(return_value=S_OK())) as checkpath_mock, \
         patch('%s.DiracILC._checkdataconsistency' % MODULE_NAME, new=Mock(return_value=S_ERROR('consistency_testerr'))) as checkconsistency_mock:
      assertDiracFailsWith(self.dilc._do_check(job_mock), 'consistency_testerr', self)
      checksb_mock.assert_called_once_with(job_mock)
      job_mock._resolveInputSandbox.assert_called_once_with(['mysandbox', 'other_value',
                                                               'sandbox_file1.txt', 'sandbox_file2.log',
                                                               'last.file'])
      job_mock._addParameter.assert_called_once_with(
          wf_mock, 'InputSandbox', 'JDL', 'resolved_file.1.txt;other_resolved_file.txt', 'Input sandbox file list')
      assertMockCalls(checkapp_mock, [('pf14081', 'myapp1v', '9.2.1'),
                                        ('pf14081', 'other_appv', '91.3'),
                                        ('pf14081', 'more_dependencies', '1'),
                                        ('pf14081', 'last_app', '0')], self)
      checkpath_mock.assert_called_once_with('path1948512895')
      checkconsistency_mock.assert_called_once_with('data1389518', 'job_sandbox13895')

  def test_checkapp(self):
    ops_mock = Mock()
    ops_mock.getValue.return_value = ''
    self.dilc.ops = ops_mock
    assertDiracFailsWith(self.dilc._checkapp('test_platform_341', 'testapp', 'v13.2'),
                          'could not find the specified software testapp_v13.2 for test_platform_341, check in CS',
                          self)
    assertMockCalls(ops_mock.getValue, [
        ('/AvailableTarBalls/test_platform_341/testapp/v13.2/TarBall', ''),
        ('/AvailableTarBalls/test_platform_341/testapp/v13.2/CVMFSPath', '')], self)

  def test_checkoutputpath_invalidchar_1(self):
    assertDiracFailsWith(self.dilc._checkoutputpath('http://www.mysitedoesnotexist3h3.abc/some/file.txt'),
                          'invalid path', self)

  def test_checkoutputpath_invalidchar_2(self):
    assertDiracFailsWith(self.dilc._checkoutputpath('/my/dir/./some/file.log'),
                          'invalid path', self)

  def test_checkoutputpath_invalidchar_3(self):
    assertDiracFailsWith(self.dilc._checkoutputpath('/my/dir/../dir2/somefile.txt'),
                          'invalid path', self)

  def test_checkoutputpath_trailing_slash(self):
    assertDiracFailsWith(self.dilc._checkoutputpath('/my/dir/myfile.txt/  '),
                          'invalid path', self)

  def test_checkdataconsistency_outputdata_sandbox_equal(self):
    assertDiracFailsWith(self.dilc._checkdataconsistency('same_item;something_else',
                                                           ['distinct_item1', 'same_item']),
                          'output data and sandbox should not contain the same thing', self)

  def test_checkdataconsistency_wildcardchar(self):
    assertDiracFailsWith(self.dilc._checkdataconsistency(
        '/test/dir/file.txt;/mydir/something/*;/file/dir/log.log',
        ['/input/sandbox.pdf', '/other/sb/file.stdhep']),
        'wildcard character in outputdata definition', self)

  def test_checkinputsb_getreplicas_notok(self):
    job_mock = Mock()
    job_mock.workflow.findParameter.return_value.getValue.return_value = '/some/file.txt;/other/some/file.stdhep;LFN:/my/dir/inputsandbox/in1.stdio;lfn:/my/dir/inputsandbox/in2.pdf'
    with patch('%s.DiracILC.getReplicas' % MODULE_NAME, new=Mock(return_value=S_ERROR('some_err'))) as replica_mock:
      assertDiracFailsWith(self.dilc.checkInputSandboxLFNs(job_mock), 'could not get replicas', self)
      replica_mock.assert_called_once_with(['/my/dir/inputsandbox/in1.stdio', '/my/dir/inputsandbox/in2.pdf'])

  def test_checkinputsb_getreplicas_fails(self):
    job_mock = Mock()
    job_mock.workflow.findParameter.return_value.getValue.return_value = '/some/file.txt;/other/some/file.stdhep;LFN:/my/dir/inputsandbox/in1.stdio;lfn:/my/dir/inputsandbox/in2.pdf'
    ret_dict = {'Failed': ['/failed/replica1', '/other/inval/replica'], 'Successful': {}}
    with patch('%s.DiracILC.getReplicas' % MODULE_NAME, new=Mock(return_value=S_OK(ret_dict))) as replica_mock:
      assertDiracFailsWith(self.dilc.checkInputSandboxLFNs(job_mock), 'failed to find replicas', self)
      replica_mock.assert_called_once_with(['/my/dir/inputsandbox/in1.stdio', '/my/dir/inputsandbox/in2.pdf'])

  def test_checkinputsb(self):
    job_mock = Mock()
    job_mock.workflow.findParameter.return_value.getValue.return_value = '/some/file.txt;/other/some/file.stdhep;LFN:/my/dir/inputsandbox/in1.stdio;lfn:/my/dir/inputsandbox/in2.pdf'
    ret_dict = {'Failed': [], 'Successful': {'/one/replica': {'SE': 'surl'}}}
    with patch('%s.DiracILC.getReplicas' % MODULE_NAME, new=Mock(return_value=S_OK(ret_dict))) as replica_mock:
      assertDiracSucceeds(self.dilc.checkInputSandboxLFNs(job_mock), self)
      replica_mock.assert_called_once_with(['/my/dir/inputsandbox/in1.stdio', '/my/dir/inputsandbox/in2.pdf'])

  def test_checkinputsb_notInputSB(self):
    job_mock = Mock()
    job_mock.workflow.findParameter.return_value = None
    assertDiracSucceeds(self.dilc.checkInputSandboxLFNs(job_mock), self)

  def test_checkinputsb_notInputSB_Value(self):
    job_mock = Mock()
    job_mock.workflow.findParameter.return_value.getValue.return_value = ''
    assertDiracSucceeds(self.dilc.checkInputSandboxLFNs(job_mock), self)

  def test_checkinputsb_noLFNs(self):
    job_mock = Mock()
    job_mock.workflow.findParameter.return_value.getValue.return_value = '/some/file.txt;/other/some/file.stdhep'
    assertDiracSucceeds(self.dilc.checkInputSandboxLFNs(job_mock), self)

  def test_checkinputsb_noRepl(self):
    job_mock = Mock()
    job_mock.workflow.findParameter.return_value.getValue.return_value = 'LFN:/some/file.txt'
    ret_dict = {'Failed': [], 'Successful': {'/some/file.txt': {'Bad-SE': 'surl'}}}

    def setOptions(*args):
      if 'SingleReplicaSEs' in args[0]:
        return ['Awesome-Disk-SE']
      if 'Minimum' in args[0]:
        return 2
      if args[0].endswith('PreferredSEs'):
        return ['Awesome-Tape-SE']

    ops_mock = Mock()
    ops_mock.getValue = setOptions
    self.dilc.ops = ops_mock

    with patch('%s.DiracILC.getReplicas' % MODULE_NAME, new=Mock(return_value=S_OK(ret_dict))) as replica_mock:
      assertDiracFailsWith(self.dilc.checkInputSandboxLFNs(job_mock), 'Not enough replicas', self)
      replica_mock.assert_called_once_with(['/some/file.txt'])

  def test_checkinputsb_goodRepl(self):
    job_mock = Mock()
    job_mock.workflow.findParameter.return_value.getValue.return_value = 'LFN:/some/file.txt'
    ret_dict = {'Failed': [], 'Successful': {'/some/file.txt': {'Awesome-Disk-SE': 'surl'}}}

    def setOptions(*args):
      if 'SingleReplicaSEs' in args[0]:
        return ['Awesome-Disk-SE']
      if 'Minimum' in args[0]:
        return 2
      if args[0].endswith('PreferredSEs'):
        return ['Awesome-Tape-SE']

    ops_mock = Mock()
    ops_mock.getValue = setOptions
    self.dilc.ops = ops_mock

    with patch('%s.DiracILC.getReplicas' % MODULE_NAME, new=Mock(return_value=S_OK(ret_dict))) as replica_mock:
      assertDiracSucceeds(self.dilc.checkInputSandboxLFNs(job_mock), self)
      replica_mock.assert_called_once_with(['/some/file.txt'])



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""ILD specific registration of file meta data.

:since: Mar 21, 2013
:author: sposs
"""

from __future__ import absolute_import
import os

from ILCDIRAC.Workflow.Modules.ModuleBase import ModuleBase
from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient

from DIRAC import S_OK, gLogger

__RCSID__ = '$Id$'
LOG = gLogger.getSubLogger(__name__)


class ILDRegisterOutputData(ModuleBase):
  """Register output data in the FC for the ILD productions."""

  def __init__(self):
    super(ILDRegisterOutputData, self).__init__()
    self.version = "ILDRegisterOutputData v1"
    self.commandTimeOut = 10 * 60
    self.enable = True
    self.prodOutputLFNs = []
    self.nbofevents = 0
    self.filecatalog = FileCatalogClient()
    self.ildconfig = ''
    self.swpackages = []

  def applicationSpecificInputs(self):
    if 'Enable' in self.step_commons:
      self.enable = self.step_commons['Enable']
      if not isinstance(self.enable, type(True)):
        LOG.warn('Enable flag set to non-boolean value %s, setting to False' % self.enable)
        self.enable = False

    if 'ProductionOutputData' in self.workflow_commons:
      self.prodOutputLFNs = self.workflow_commons['ProductionOutputData'].split(";")
    else:
      self.prodOutputLFNs = []

    if 'SoftwarePackages' in self.workflow_commons:
      self.swpackages = self.workflow_commons['SoftwarePackages'].split(";")

    self.nbofevents = self.NumberOfEvents  # comes from ModuleBase
    if 'ILDConfigPackage' in self.workflow_commons:
      self.ildconfig = self.workflow_commons['ILDConfigPackage']
    return S_OK('Parameters resolved')

  def execute(self):
    LOG.info('Initializing %s' % self.version)
    result = self.resolveInputVariables()
    if not result['OK']:
      LOG.error("Failed to resolve input parameters:", result['Message'])
      return result

    if not self.workflowStatus['OK'] or not self.stepStatus['OK']:
      LOG.verbose('Workflow status = %s, step status = %s' % (self.workflowStatus['OK'], self.stepStatus['OK']))
      return S_OK('No registration of output data metadata attempted')

    if len(self.prodOutputLFNs) == 0:
      LOG.info('No production data found, so no metadata registration to be done')
      return S_OK("No files' metadata to be registered")

    LOG.verbose("Will try to set the metadata for the following files: \n %s" % "\n".join(self.prodOutputLFNs))

    # TODO: What meta data should be stored at file level?

    for files in self.prodOutputLFNs:
      meta = {}

      if self.nbofevents:
        nbevts = {}
        nbevts['NumberOfEvents'] = self.nbofevents
        if 'file_number_of_event_relation' in self.workflow_commons:
          if os.path.basename(files) in self.workflow_commons['file_number_of_event_relation']:
            nbevts['NumberOfEvents'] = self.workflow_commons['file_number_of_event_relation'][os.path.basename(files)]
        meta.update(nbevts)

      if 'CrossSection' in self.inputdataMeta:
        xsec = {'CrossSection': self.inputdataMeta['CrossSection']}
        meta.update(xsec)

      if 'CrossSectionError' in self.inputdataMeta:
        xsec = {'CrossSectionError': self.inputdataMeta['CrossSectionError']}
        meta.update(xsec)

      if 'GenProcessID' in self.inputdataMeta:
        fmeta = {'GenProcessID': self.inputdataMeta['GenProcessID']}
        meta.update(fmeta)

      if 'GenProcessType' in self.inputdataMeta:
        fmeta = {'GenProcessType': self.inputdataMeta['GenProcessType']}
        meta.update(fmeta)

      if 'GenProcessName' in self.inputdataMeta:
        fmeta = {'GenProcessName': self.inputdataMeta['GenProcessName']}
        meta.update(fmeta)

      if 'Luminosity' in self.inputdataMeta:
        fmeta = {'Luminosity': self.inputdataMeta['Luminosity']}
        meta.update(fmeta)

      if 'BeamParticle1' in self.inputdataMeta:
        fmeta = {'BeamParticle1': self.inputdataMeta['BeamParticle1'],
                 'BeamParticle2': self.inputdataMeta['BeamParticle2']}
        meta.update(fmeta)

      if 'PolarizationB1' in self.inputdataMeta:
        fmeta = {'PolarizationB1': self.inputdataMeta['PolarizationB1'],
                 'PolarizationB2': self.inputdataMeta['PolarizationB2']}
        meta.update(fmeta)

      if self.ildconfig:
        fmeta = {'ILDConfig': self.ildconfig}
        meta.update(fmeta)

      if self.WorkflowStartFrom:
        meta.update({"FirstEventFromInput": self.WorkflowStartFrom})

      if self.enable:
        res = self.filecatalog.setMetadata(files, meta)
        if not res['OK']:
          LOG.error('Could not register metadata:', res['Message'])
          return res
      LOG.info("Registered %s with tags %s" % (files, meta))

      # Now, set the ancestors
      if self.InputData:
        inputdata = self.InputData
        if self.enable:
          res = self.filecatalog.addFileAncestors({files: {'Ancestors': inputdata}})
          if not res['OK']:
            LOG.error('Registration of Ancestors failed for:', str(files))
            LOG.error('because of', res['Message'])
            return res

    return S_OK('Output data metadata registered in catalog')



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Module to remove software. Not used if using the ProcessProductionSystem.

:author: Stephane Poss
:since: Jul 14, 2011
"""
from __future__ import absolute_import
__RCSID__ = "$Id$"

from ILCDIRAC.Core.Utilities.CombinedSoftwareInstallation import getLocalAreaLocation, getSharedAreaLocation
from DIRAC.ConfigurationSystem.Client.Helpers.Operations import Operations

from DIRAC import S_OK, S_ERROR, gLogger
import os
import shutil


class RemoveSoft(object):
  """Utility to remove software from a site."""

  def __init__(self):
    self.softs = ''
    self.apps = []
    self.log = gLogger.getSubLogger(__name__)
    self.platform = ''
    self.step_commons = {}
    self.workflow_commons = {}
    self.ops = Operations()

  def execute(self):
    """Look in folders (Shared Area and Local Area) and try ot remove the applications specified."""
    self.softs = self.step_commons.get('Apps', None)
    if not self.softs:
      return S_ERROR('Applications to remove were not defined')

    self.platform = self.workflow_commons.get('Platform', None)
    if not self.platform:
      return S_ERROR('Platform, formerly known as SystemConfig not defined')

    self.softs.rstrip(";")
    self.apps = self.softs.split(';')
    self.log.info("Will delete %s" % self.apps)
    failed = []
    for app in self.apps:
      if not app:
        continue

      appname = app.split(".")[0]
      appversion = app.split(".")[1]
      appDir = self.ops.getValue('/AvailableTarBalls/%s/%s/%s/TarBall' % (self.platform, appname, appversion), '')
      appDir = appDir.replace(".tgz", "").replace(".tar.gz", "")
      mySoftwareRoot = ''
      localArea = getLocalAreaLocation()
      sharedArea = getSharedAreaLocation()
      if os.path.exists('%s%s%s' % (localArea, os.sep, appDir)):
        mySoftwareRoot = localArea
      elif os.path.exists('%s%s%s' % (sharedArea, os.sep, appDir)):
        mySoftwareRoot = sharedArea
      else:
        self.log.error('%s: Could not find neither local area not shared area install' % app)
        continue
      myappDir = os.path.join(mySoftwareRoot, appDir)

      # Hacky hack needed when the DB was in parallel to the Mokka version
      if appname.lower() == 'mokka':
        dbloc = os.path.join(mySoftwareRoot, "CLICMokkaDB.sql")
        if os.path.exists(dbloc):
          try:
            os.remove(dbloc)
          except OSError as x:
            self.log.error("Could not delete SQL DB file : %s" % (str(x)))
      if os.path.isdir(myappDir):
        try:
          shutil.rmtree(myappDir)
        except OSError as x:
          self.log.error("Could not delete %s : %s" % (app, str(x)))
          failed.append(app)
      else:
        try:
          os.remove(myappDir)
        except OSError as x:
          self.log.error("Could not delete %s: %s" % (myappDir, str(x)))

    if len(failed):
      return S_ERROR("Failed deleting applications %s" % failed)
    self.log.info("Successfully deleted %s" % self.apps)
    return S_OK()



#!/usr/bin/env python
#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Test the SLICPandora module."""

from __future__ import absolute_import
import unittest
import sys
from mock import patch, mock_open, MagicMock as Mock

from DIRAC import S_OK, S_ERROR
from Tests.Utilities.GeneralUtils import assertEqualsImproved, \
    assertDiracFailsWith, assertDiracSucceeds, assertDiracSucceedsWith, \
    assertDiracSucceedsWith_equals, assertMockCalls

__RCSID__ = "$Id$"

MODULE_NAME = 'ILCDIRAC.Workflow.Modules.SLICPandoraAnalysis'


class TestSLICPandora(unittest.TestCase):
  """Test the SLICPandora module."""

  def setUp(self):
    # Mock out modules that spawn other threads
    mocked_modules = {'DIRAC.DataManagementSystem.Client.DataManager': Mock()}
    self.module_patcher = patch.dict(sys.modules, mocked_modules)
    self.module_patcher.start()
    from ILCDIRAC.Workflow.Modules.SLICPandoraAnalysis import SLICPandoraAnalysis
    self.spa = SLICPandoraAnalysis()
    self.spa.platform = 'myTestPlatform'
    self.spa.applicationLog = 'applogFile.test'

  def tearDown(self):
    self.module_patcher.stop()

  def test_applicationSpecificInputs(self):
    assertDiracSucceedsWith_equals(self.spa.applicationSpecificInputs(),
                                    'Parameters resolved', self)
    assertEqualsImproved(self.spa.pandorasettings, 'PandoraSettings.xml', self)
    assertEqualsImproved(self.spa.InputFile, [], self)
    assertEqualsImproved(self.spa.InputData, [], self)

  def test_applicationSpecificInputs_1(self):
    self.spa.pandorasettings = 'something'
    self.spa.InputData = ['myfile.slcio', 'ignorethisfile', 'testPart1.slcio',
                           'myarchive.slcio.tar.gz']
    assertDiracSucceedsWith_equals(self.spa.applicationSpecificInputs(),
                                    'Parameters resolved', self)
    assertEqualsImproved(self.spa.InputFile,
                          ['myfile.slcio', 'testPart1.slcio',
                            'myarchive.slcio.tar.gz'], self)

  def test_runit(self):
    exists_dict = {'testmodelDetector_pandora.xml': False,
                    '/secret/dir/testmodelDetector_pandora.xml': True,
                    '/secret/dir/testmodelDetector.zip': False,
                    'SLICPandora__Run_465.sh': True, './lib': False,
                    'applogFile.test': True}
    self.spa.detectorxml = '/secret/dir/testmodelDetector_pandora.xml'
    self.spa.InputFile = 'testInput.file'
    self.spa.STEP_NUMBER = 465
    ops_mock = Mock()
    ops_mock.getValue.return_value = ['detector_1_url_throw_error.pdf',
                                       'detector2_unzip_fails.xml',
                                       'working_detector_v4.xml']
    self.spa.ops = ops_mock
    mo = mock_open(read_data='some_log_data\nsuccessful finish :)')
    with patch('%s.getEnvironmentScript' % MODULE_NAME, new=Mock(return_value=S_OK('myenvscriptpathtestme'))), \
             patch('%s.resolveIFpaths' % MODULE_NAME, new=Mock(return_value=S_OK(['ruinonslcio.test']))), \
             patch('%s.unzip_file_into_dir' % MODULE_NAME, new=Mock(side_effect=[OSError('unzipping failed'), True])) as unzip_mock, \
             patch('%s.os.unlink' % MODULE_NAME) as unlink_mock, \
             patch('%s.os.chmod' % MODULE_NAME) as chmod_mock, \
             patch('%s.open' % MODULE_NAME, mo) as open_mock, \
             patch('%s.six.moves.urllib.request.urlretrieve' % MODULE_NAME,
                   new=Mock(side_effect=[IOError('my_test_ioerr'), True, True])), \
             patch('%s.shellCall' % MODULE_NAME, new=Mock(return_value=S_OK([0, 'Disabled execution', '']))) as shell_mock, \
             patch('%s.os.getcwd' % MODULE_NAME, new=Mock(return_value='/my/curdir/test')), \
             patch('%s.os.path.exists' % MODULE_NAME, new=Mock(side_effect=lambda path: exists_dict[path])) as exists_mock, patch('%s.os.remove' % MODULE_NAME) as remove_mock:
      assertDiracSucceeds(self.spa.runIt(), self)
      assertMockCalls(exists_mock, ['testmodelDetector_pandora.xml', '/secret/dir/testmodelDetector.zip',
                                      '/secret/dir/testmodelDetector.zip',
                                      '/secret/dir/testmodelDetector_pandora.xml', 'SLICPandora__Run_465.sh',
                                      './lib', 'applogFile.test', 'applogFile.test'], self)
      assertMockCalls(open_mock, [('/secret/dir/testmodelDetector.zip', 'rb'), ('SLICPandora__Run_465.sh', 'w'),
                                    ('applogFile.test', 'r')], self, only_these_calls=False)
      mo = mo()
      assertMockCalls(unzip_mock, [(mo, '/my/curdir/test'), (mo, '/my/curdir/test')], self)
      open_mock = open_mock()
      assertMockCalls(open_mock.write, [
          '#!/bin/bash \n', '#####################################################################\n',
          '# Dynamically generated script to run a production or analysis job. #\n',
          '#####################################################################\n',
          'source myenvscriptpathtestme\n', 'declare -x file=./Settings/\n',
          '\nif [ -e "${file}" ]\nthen\n   declare -x PANDORASETTINGS=$file\nelse\n  if [ -d "${PANDORASETTINGSDIR}" ]\n  then\n    cp $PANDORASETTINGSDIR/*.xml .\n    declare -x PANDORASETTINGS=\n  fi\nfi\nif [ ! -e "${PANDORASETTINGS}" ]\nthen\n  echo "Missing PandoraSettings file"\n  exit 1\nfi\n',
          'echo =============================\n', 'echo PATH is \n', 'echo $PATH | tr ":" "\n"  \n',
          'echo ==============\n', 'echo =============================\n', 'echo LD_LIBRARY_PATH is \n',
          'echo $LD_LIBRARY_PATH | tr ":" "\n"\n', 'echo ============================= \n',
          'env | sort >> localEnv.log\n',
          'PandoraFrontend -g /secret/dir/testmodelDetector_pandora.xml -c $PANDORASETTINGS -i ruinonslcio.test -o  -r 0 \n',
          'declare -x appstatus=$?\n', 'exit $appstatus\n'], self)
      assertMockCalls(remove_mock, ['SLICPandora__Run_465.sh', 'applogFile.test'], self)
      chmod_mock.assert_called_once_with('SLICPandora__Run_465.sh', 0o755)
      shell_mock.assert_called_once_with(0, 'sh -c "./SLICPandora__Run_465.sh"',
                                          callbackFunction=self.spa.redirectLogOutput,
                                          bufferLimit=20971520)
      unlink_mock.assert_called_once_with('/secret/dir/testmodelDetector.zip')

  def test_runit_noplatform(self):
    self.spa.platform = None
    result = self.spa.runIt()
    assertDiracFailsWith(result, 'no ilc platform selected', self)

  def test_runit_nolog(self):
    self.spa.applicationLog = None
    result = self.spa.runIt()
    assertDiracFailsWith(result, 'no log file provided', self)

  def test_runit_workflowstatus_bad(self):
    self.spa.workflowStatus = S_ERROR('workflow_err_testme')
    assertDiracSucceedsWith_equals(self.spa.runIt(),
                                    'SLIC Pandora should not proceed as previous step did not end properly',
                                    self)

  def test_runit_stepstatus_bad(self):
    self.spa.stepStatus = S_ERROR('step_err_testme')
    assertDiracSucceedsWith_equals(self.spa.runIt(),
                                    'SLIC Pandora should not proceed as previous step did not end properly',
                                    self)

  def test_runit_getsoftwarefolder_fails(self):
    with patch('%s.getEnvironmentScript' % MODULE_NAME, new=Mock(return_value=S_ERROR('get_envscriptI_test_error'))) as getsoft_mock:
      assertDiracFailsWith(self.spa.runIt(), 'get_envscriptI_test_error', self)
      getsoft_mock.assert_called_once_with('myTestPlatform', 'SLICPandora', '', self.spa.getEnvScript)

  def test_runit_resolveIFPaths_fails(self):
    with patch('%s.getEnvironmentScript' % MODULE_NAME, new=Mock(return_value=S_OK('myenvscriptpathtestme'))) as getsoft_mock, \
        patch('%s.resolveIFpaths' % MODULE_NAME, new=Mock(return_value=S_ERROR('slcio_err_test'))), \
        patch('%s.SLICPandoraAnalysis.setApplicationStatus' % MODULE_NAME) as appstat_mock:
      assertDiracFailsWith(self.spa.runIt(), 'missing slcio file', self)
      getsoft_mock.assert_called_once_with('myTestPlatform', 'SLICPandora', '',
                                            self.spa.getEnvScript)
      appstat_mock.assert_called_once_with('SLICPandora: missing slcio file')

  def test_runit_no_detectormodel_found(self):
    exists_dict = {'testmodelDetector.xml': True, '/secret/dir/testmodelDetector.xml': False}
    self.spa.detectorxml = '/secret/dir/testmodelDetector.xml'
    self.spa.InputFile = 'testInput.file'
    with patch('%s.getEnvironmentScript' % MODULE_NAME, new=Mock(return_value=S_OK('myenvscriptpathtestme'))), \
             patch('%s.resolveIFpaths' % MODULE_NAME, new=Mock(return_value=S_OK(['ruinonslcio.test']))) as resolve_mock, \
             patch('%s.os.path.exists' % MODULE_NAME, new=Mock(side_effect=lambda path: exists_dict[path])) as exists_mock:
      assertDiracFailsWith(self.spa.runIt(),
                            'detector model xml was not found, exiting', self)
      assertMockCalls(exists_mock, ['testmodelDetector.xml', '/secret/dir/testmodelDetector.xml'], self)
      resolve_mock.assert_called_once_with('testInput.file')

  def test_runit_no_applog_created(self):
    exists_dict = {'testmodelDetector_pandora.xml': False,
                    '/secret/dir/testmodelDetector_pandora.xml': True,
                    '/secret/dir/testmodelDetector.zip': True,
                    'SLICPandora__Run_465.sh': True, './lib': True,
                    'applogFile.test': True}
    self.spa.detectorxml = '/secret/dir/testmodelDetector_pandora.xml'
    self.spa.InputFile = 'testInput.file'
    self.spa.STEP_NUMBER = 465
    mo = mock_open()

    def replace_exists(path):
      """Mock exists implementation."""
      if path == 'applogFile.test':
        result = exists_dict[path]
        exists_dict[path] = False
        return result
      else:
        return exists_dict[path]
    with patch('%s.getEnvironmentScript' % MODULE_NAME, new=Mock(return_value=S_OK('myenvscriptpathtestme'))), \
             patch('%s.resolveIFpaths' % MODULE_NAME, new=Mock(return_value=S_OK(['ruinonslcio.test']))), \
             patch('%s.unzip_file_into_dir' % MODULE_NAME) as unzip_mock, \
             patch('%s.os.unlink' % MODULE_NAME) as unlink_mock, \
             patch('%s.os.chmod' % MODULE_NAME) as chmod_mock, \
             patch('%s.open' % MODULE_NAME, mo) as open_mock, \
             patch('%s.shellCall' % MODULE_NAME, new=Mock(return_value=S_OK(['something']))) as shell_mock, \
             patch('%s.os.getcwd' % MODULE_NAME, new=Mock(return_value='/my/curdir/test')), \
             patch('%s.os.path.exists' % MODULE_NAME, new=Mock(side_effect=replace_exists)) as exists_mock, patch('%s.os.remove' % MODULE_NAME) as remove_mock:
      assertDiracFailsWith(self.spa.runIt(),
                            'SLICPandora did not produce the expected log',
                            self)
      assertMockCalls(exists_mock, ['testmodelDetector_pandora.xml', '/secret/dir/testmodelDetector.zip',
                                      '/secret/dir/testmodelDetector.zip',
                                      '/secret/dir/testmodelDetector_pandora.xml', 'SLICPandora__Run_465.sh',
                                      './lib', 'applogFile.test', 'applogFile.test'], self)
      assertMockCalls(open_mock, [('/secret/dir/testmodelDetector.zip', 'rb'), ('SLICPandora__Run_465.sh', 'w')],
                        self, only_these_calls=False)
      mo = mo()
      unzip_mock.assert_called_once_with(mo, '/my/curdir/test')
      open_mock = open_mock()
      assertMockCalls(open_mock.write, [
          '#!/bin/bash \n', '#####################################################################\n',
          '# Dynamically generated script to run a production or analysis job. #\n',
          '#####################################################################\n',
          'source myenvscriptpathtestme\n', 'declare -x file=./Settings/\n',
          '\nif [ -e "${file}" ]\nthen\n   declare -x PANDORASETTINGS=$file\nelse\n  if [ -d "${PANDORASETTINGSDIR}" ]\n  then\n    cp $PANDORASETTINGSDIR/*.xml .\n    declare -x PANDORASETTINGS=\n  fi\nfi\nif [ ! -e "${PANDORASETTINGS}" ]\nthen\n  echo "Missing PandoraSettings file"\n  exit 1\nfi\n',
          'echo =============================\n', 'echo PATH is \n',
          'echo $PATH | tr ":" "\n"  \n', 'echo ==============\n',
          'declare -x LD_LIBRARY_PATH=./lib:$LD_LIBRARY_PATH\n', 'echo =============================\n',
          'echo LD_LIBRARY_PATH is \n', 'echo $LD_LIBRARY_PATH | tr ":" "\n"\n',
          'echo ============================= \n', 'env | sort >> localEnv.log\n',
          'PandoraFrontend -g /secret/dir/testmodelDetector_pandora.xml -c $PANDORASETTINGS -i ruinonslcio.test -o  -r 0 \n',
          'declare -x appstatus=$?\n', 'exit $appstatus\n'], self)
      assertMockCalls(remove_mock, ['SLICPandora__Run_465.sh', 'applogFile.test'], self)
      chmod_mock.assert_called_once_with('SLICPandora__Run_465.sh', 0o755)
      shell_mock.assert_called_once_with(0, 'sh -c "./SLICPandora__Run_465.sh"',
                                          callbackFunction=self.spa.redirectLogOutput,
                                          bufferLimit=20971520)
      self.assertFalse(unlink_mock.called)

  def test_runit_unzip_fails_nodetectorURL(self):
    exists_dict = {'testmodelDetector_pandora.xml': False,
                    '/secret/dir/testmodelDetector_pandora.xml': True,
                    '/secret/dir/testmodelDetector.zip': True,
                    'SLICPandora__Run_465.sh': False, './lib': False,
                    'applogFile.test': True}
    self.spa.detectorxml = '/secret/dir/testmodelDetector_pandora.xml'
    self.spa.InputFile = 'testInput.file'
    self.spa.STEP_NUMBER = 465
    ops_mock = Mock()
    ops_mock.getValue.return_value = ['']
    self.spa.ops = ops_mock
    mo = mock_open()

    def replace_exists(path):
      """Mock exists implementation."""
      if path == '/secret/dir/testmodelDetector.zip':
        result = exists_dict[path]
        exists_dict[path] = False
        return result
      else:
        return exists_dict[path]
    with patch('%s.getEnvironmentScript' % MODULE_NAME, new=Mock(return_value=S_OK('myenvscriptpathtestme'))), \
             patch('%s.resolveIFpaths' % MODULE_NAME, new=Mock(return_value=S_OK(['ruinonslcio.test']))), \
             patch('%s.unzip_file_into_dir' % MODULE_NAME, side_effect=OSError('unable to unzip_testme')) as unzip_mock, \
             patch('%s.os.unlink' % MODULE_NAME) as unlink_mock, \
             patch('%s.os.chmod' % MODULE_NAME) as chmod_mock, \
             patch('%s.open' % MODULE_NAME, mo) as open_mock, \
             patch('%s.shellCall' % MODULE_NAME, new=Mock(return_value=S_OK(['something']))) as shell_mock, \
             patch('%s.os.getcwd' % MODULE_NAME, new=Mock(return_value='/my/curdir/test')), \
             patch('%s.os.path.exists' % MODULE_NAME, new=Mock(side_effect=replace_exists)) as exists_mock, patch('%s.os.remove' % MODULE_NAME) as remove_mock:
      assertDiracFailsWith(self.spa.runIt(),
                            'could not find in cs the url for detector model',
                            self)
      open_mock.assert_called_once_with('/secret/dir/testmodelDetector.zip', 'rb')
      assertMockCalls(exists_mock, ['testmodelDetector_pandora.xml', '/secret/dir/testmodelDetector.zip',
                                      '/secret/dir/testmodelDetector.zip'], self)
      unzip_mock.assert_called_once_with(mo(), '/my/curdir/test')
      open_mock = open_mock()
      self.assertFalse(open_mock.write.called)
      self.assertFalse(remove_mock.called)
      self.assertFalse(chmod_mock.called)
      self.assertFalse(shell_mock.called)
      unlink_mock.assert_called_once_with('/secret/dir/testmodelDetector.zip')

  def test_runit_oldversion(self):
    exists_dict = {'secret/dir/mySuperDetector_pandora.xml': False,
                    'secret/dir/mySuperDetector.zip': False,
                    '/my/curdir/test/secret/dir/mySuperDetector_pandora.xml': True,
                    'SLICPandora_V2_Run_465.sh': False, './lib': False,
                    'applogFile.test': False}
    self.spa.detectorxml = 'secret/dir/mySuperDetector'
    self.spa.applicationVersion = 'V2'
    self.spa.InputFile = 'testInput.file'
    self.spa.STEP_NUMBER = 465
    ops_mock = Mock()
    ops_mock.getValue.return_value = ['http://some_url/']
    self.spa.ops = ops_mock
    mo = mock_open(read_data='some_log_data\n Missing PandoraSettings file, but ignore this error :)')

    def replace_exists(path):
      """Mock implementation of exists."""
      result = exists_dict[path]
      if path == 'secret/dir/mySuperDetector.zip' or path == 'applogFile.test':
        exists_dict[path] = True
      return result
    with patch('%s.getEnvironmentScript' % MODULE_NAME, new=Mock(return_value=S_OK('myenvscriptpathtestme'))), \
             patch('%s.resolveIFpaths' % MODULE_NAME, new=Mock(return_value=S_OK(['ruinonslcio.test']))), \
             patch('%s.unzip_file_into_dir' % MODULE_NAME) as unzip_mock, \
             patch('%s.os.unlink' % MODULE_NAME) as unlink_mock, \
             patch('%s.os.chmod' % MODULE_NAME) as chmod_mock, \
             patch('%s.open' % MODULE_NAME, mo) as open_mock, \
             patch('%s.six.moves.urllib.request.urlretrieve' % MODULE_NAME) as urlretrieve_mock, \
             patch('%s.shellCall' % MODULE_NAME, new=Mock(return_value=S_OK([0, 'Disabled execution', '']))) as shell_mock, \
             patch('%s.os.getcwd' % MODULE_NAME, new=Mock(return_value='/my/curdir/test')), \
             patch('%s.os.path.exists' % MODULE_NAME, new=Mock(side_effect=replace_exists)) as exists_mock, patch('%s.os.remove' % MODULE_NAME) as remove_mock:
      assertDiracSucceedsWith(self.spa.runIt(), 'SLICPandora V2 Successful', self)
      assertMockCalls(exists_mock, ['secret/dir/mySuperDetector.zip', 'secret/dir/mySuperDetector.zip',
                                      '/my/curdir/test/secret/dir/mySuperDetector_pandora.xml',
                                      'SLICPandora_V2_Run_465.sh', './lib', 'applogFile.test',
                                      'applogFile.test'], self)
      assertMockCalls(open_mock, [('SLICPandora_V2_Run_465.sh', 'w'), ('applogFile.test', 'r')],
                       self, only_these_calls=False)
      mo = mo()
      open_mock = open_mock()
      assertMockCalls(open_mock.write, [
          '#!/bin/bash \n', '#####################################################################\n',
          '# Dynamically generated script to run a production or analysis job. #\n',
          '#####################################################################\n',
          'source myenvscriptpathtestme\n', 'declare -x file=./Settings/\n',
          '\nif [ -e "${file}" ]\nthen\n   declare -x PANDORASETTINGS=$file\nelse\n  if [ -d "${PANDORASETTINGSDIR}" ]\n  then\n    cp $PANDORASETTINGSDIR/*.xml .\n    declare -x PANDORASETTINGS=\n  fi\nfi\nif [ ! -e "${PANDORASETTINGS}" ]\nthen\n  echo "Missing PandoraSettings file"\n  exit 1\nfi\n',
          'echo =============================\n', 'echo PATH is \n', 'echo $PATH | tr ":" "\n"  \n',
          'echo ==============\n', 'echo =============================\n', 'echo LD_LIBRARY_PATH is \n',
          'echo $LD_LIBRARY_PATH | tr ":" "\n"\n', 'echo ============================= \n',
          'env | sort >> localEnv.log\n',
          'PandoraFrontend /my/curdir/test/secret/dir/mySuperDetector_pandora.xml $PANDORASETTINGS ruinonslcio.test  0 \n',
          'declare -x appstatus=$?\n', 'exit $appstatus\n'], self)
      chmod_mock.assert_called_once_with('SLICPandora_V2_Run_465.sh', 0o755)
      shell_mock.assert_called_once_with(0, 'sh -c "./SLICPandora_V2_Run_465.sh"',
                                          callbackFunction=self.spa.redirectLogOutput,
                                          bufferLimit=20971520)
      self.assertFalse(urlretrieve_mock.called)
      self.assertFalse(unzip_mock.called)
      self.assertFalse(unlink_mock.called)
      self.assertFalse(remove_mock.called)

  def test_getenvscript(self):
    def replace_abspath(path):
      """Mock abspath implementation."""
      if path == 'SLICPandora.sh':
        return '/abs/test/path/SLICPandora.sh'
      else:
        sys.exit()
    exists_dict = {'PandoraFrontend': True}
    with patch('%s.getSoftwareFolder' % MODULE_NAME, new=Mock(return_value=S_OK('/my/dir/test/me'))) as getsoft_mock, \
         patch('%s.removeLibc' % MODULE_NAME) as removelib_mock, \
         patch('%s.getNewLDLibs' % MODULE_NAME, new=Mock(return_value='/new/ldpath')) as getlib_mock, \
         patch('%s.getNewPATH' % MODULE_NAME, new=Mock(return_value='/new/test/path')) as getpath_mock, \
         patch('%s.open' % MODULE_NAME, mock_open()) as open_mock:
      with patch('%s.os.path.exists' % MODULE_NAME, new=Mock(side_effect=lambda path: exists_dict[path])) as exists_mock, \
          patch('%s.os.chmod' % MODULE_NAME) as chmod_mock, \
          patch('%s.os.path.abspath' % MODULE_NAME, new=Mock(side_effect=replace_abspath)) as abspath_mock:
        result = self.spa.getEnvScript('mytestsysconfig', 'SLIC Pandora', 'V2')
        exists_mock.assert_called_once_with('PandoraFrontend')
        abspath_mock.assert_called_once_with('SLICPandora.sh')
        assertDiracSucceedsWith_equals(result, '/abs/test/path/SLICPandora.sh', self)
        chmod_mock.assert_called_once_with('SLICPandora.sh', 0o755)
      getsoft_mock.assert_called_once_with('mytestsysconfig', 'SLIC Pandora', 'V2')
      removelib_mock.assert_called_once_with('/my/dir/test/me/LDLibs')
      getlib_mock.assert_called_once_with('mytestsysconfig', 'SLIC Pandora', 'V2')
      getpath_mock.assert_called_once_with('mytestsysconfig', 'SLIC Pandora', 'V2')
      open_mock.assert_any_call('SLICPandora.sh', 'w')
      open_mock = open_mock()
      assertMockCalls(open_mock.write, [
          '#!/bin/sh \n', '############################################################\n',
          '# Dynamically generated script to get the SLICPandora env. #\n',
          '############################################################\n',
          "declare -x PATH=/new/test/path:$PATH\n", 'declare -x ROOTSYS=/my/dir/test/me/ROOT\n',
          'declare -x LD_LIBRARY_PATH=$ROOTSYS/lib:/my/dir/test/me/LDLibs:/new/ldpath\n',
          'declare -x PANDORASETTINGSDIR=/my/dir/test/me/Settings\n', "declare -x PATH=.:$PATH\n"], self)

  def test_getenvscript_getsoftware_fails(self):
    with patch('%s.getSoftwareFolder' % MODULE_NAME, new=Mock(return_value=S_ERROR('getsoftware_test_err'))), \
        patch('%s.SLICPandoraAnalysis.setApplicationStatus' % MODULE_NAME, new=Mock(return_value=S_ERROR())):
      result = self.spa.getEnvScript('mytestsysconfig', 'SLIC Pandora', 'V2')
      assertDiracFailsWith(result, 'getsoftware_test_err', self)

  def test_getenvscript_other_prefixpath(self):
    def replace_abspath(path):
      """Mock implementation of os.path.abspath."""
      if path == 'SLICPandora.sh':
        return '/abs/test/path/SLICPandora.sh'
      else:
        sys.exit()
    exists_dict = {'PandoraFrontend': False, '/my/dir/test/me/Executable/PandoraFrontend': True}
    with patch('%s.getSoftwareFolder' % MODULE_NAME, new=Mock(return_value=S_OK('/my/dir/test/me'))) as getsoft_mock, \
         patch('%s.removeLibc' % MODULE_NAME) as removelib_mock, \
         patch('%s.getNewLDLibs' % MODULE_NAME, new=Mock(return_value='/new/ldpath')) as getlib_mock, \
         patch('%s.getNewPATH' % MODULE_NAME, new=Mock(return_value='/new/test/path')) as getpath_mock, \
         patch('%s.open' % MODULE_NAME, mock_open()) as open_mock:
      with patch('%s.os.path.exists' % MODULE_NAME, new=Mock(side_effect=lambda path: exists_dict[path])) as exists_mock, \
          patch('%s.os.chmod' % MODULE_NAME) as chmod_mock, \
          patch('%s.os.path.abspath' % MODULE_NAME, new=Mock(side_effect=replace_abspath)) as abspath_mock:
        result = self.spa.getEnvScript('mytestsysconfig', 'SLIC Pandora', 'V2')
        chmod_mock.assert_called_once_with('SLICPandora.sh', 0o755)
        assertMockCalls(exists_mock, ['PandoraFrontend', '/my/dir/test/me/Executable/PandoraFrontend'], self)
        abspath_mock.assert_called_once_with('SLICPandora.sh')
      assertDiracSucceedsWith_equals(result, '/abs/test/path/SLICPandora.sh', self)
      getsoft_mock.assert_called_once_with('mytestsysconfig', 'SLIC Pandora', 'V2')
      removelib_mock.assert_called_once_with('/my/dir/test/me/LDLibs')
      getlib_mock.assert_called_once_with('mytestsysconfig', 'SLIC Pandora', 'V2')
      getpath_mock.assert_called_once_with('mytestsysconfig', 'SLIC Pandora', 'V2')
      open_mock.assert_any_call('SLICPandora.sh', 'w')
      open_mock = open_mock()
      assertMockCalls(open_mock.write, [
          '#!/bin/sh \n', '############################################################\n',
          '# Dynamically generated script to get the SLICPandora env. #\n',
          '############################################################\n',
          "declare -x PATH=/new/test/path:$PATH\n", 'declare -x ROOTSYS=/my/dir/test/me/ROOT\n',
          'declare -x LD_LIBRARY_PATH=$ROOTSYS/lib:/my/dir/test/me/LDLibs:/new/ldpath\n',
          'declare -x PANDORASETTINGSDIR=/my/dir/test/me/Settings\n', "declare -x PATH=/my/dir/test/me/Executable:$PATH\n"], self)

  def test_getenvscript_pandorafrontend_missing(self):
    exists_dict = {'PandoraFrontend': False, '/my/dir/test/me/Executable/PandoraFrontend': False}
    with patch('%s.getSoftwareFolder' % MODULE_NAME, new=Mock(return_value=S_OK('/my/dir/test/me'))) as getsoft_mock, \
         patch('%s.removeLibc' % MODULE_NAME) as removelib_mock, \
         patch('%s.getNewLDLibs' % MODULE_NAME, new=Mock(return_value='/new/ldpath')) as getlib_mock, \
         patch('%s.getNewPATH' % MODULE_NAME, new=Mock(return_value='/new/test/path')) as getpath_mock, \
         patch('%s.open' % MODULE_NAME, mock_open()) as open_mock, \
         patch('%s.os.path.exists' % MODULE_NAME, new=Mock(side_effect=lambda path: exists_dict[path])) as exists_mock, \
         patch('%s.os.chmod' % MODULE_NAME) as chmod_mock:
      result = self.spa.getEnvScript('mytestsysconfig', 'SLIC Pandora', 'V2')
      assertDiracFailsWith(result, 'missing pandorafrontend binary', self)
      getsoft_mock.assert_called_once_with('mytestsysconfig', 'SLIC Pandora', 'V2')
      removelib_mock.assert_called_once_with('/my/dir/test/me/LDLibs')
      getlib_mock.assert_called_once_with('mytestsysconfig', 'SLIC Pandora', 'V2')
      getpath_mock.assert_called_once_with('mytestsysconfig', 'SLIC Pandora', 'V2')
      self.assertFalse(open_mock.called)
      self.assertFalse(chmod_mock.called)
      assertMockCalls(exists_mock, ['PandoraFrontend', '/my/dir/test/me/Executable/PandoraFrontend'], self)



#!/usr/local/env python
#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Test DDSim module."""

from __future__ import print_function
from __future__ import absolute_import
import inspect
import unittest
from mock import create_autospec, patch, MagicMock as Mock

from DIRAC import gLogger, S_OK, S_ERROR
from ILCDIRAC.Interfaces.API.NewInterface.Applications import DDSim
from Tests.Utilities.GeneralUtils import assertEqualsImproved, assertDiracFailsWith, \
    assertDiracSucceeds

__RCSID__ = "$Id$"

MODULE_NAME = 'ILCDIRAC.Interfaces.API.NewInterface.Applications.DDSim'

gLogger.setLevel("DEBUG")
gLogger.showHeaders(True)

# pylint: disable=protected-access


class DDSimTestCase(unittest.TestCase):
  """Base class for the DDSim test cases."""

  def setUp(self):
    """set up the objects."""
    self.dds = DDSim({})

  def test_setrandomseed(self):
    self.assertFalse(self.dds._errorDict)
    self.dds.setRandomSeed(89421)
    self.assertFalse(self.dds._errorDict)
    assertEqualsImproved(self.dds.randomSeed, 89421, self)

  def test_setrandomseed_fails(self):
    self.assertFalse(self.dds._errorDict)
    self.dds.setRandomSeed(['abc'])
    self.assertIn('_checkArgs', self.dds._errorDict)

  def test_addextraparticles_pass(self):
    """Test if API write correctly."""
    self.assertFalse(self.dds._errorDict)
    self.dds.setExtraParticles([[25, "Maribor", 2, 1500., 10., 100.]])
    self.assertFalse(self.dds._errorDict)
    assertEqualsImproved(self.dds.extraParticles, [[25, "Maribor", 2, 1500., 10., 100.]], self)

  def test_addextraparticles_pass2(self):
    """Test if API write correctly."""
    self.assertFalse(self.dds._errorDict)
    self.dds.setExtraParticles("25 Maribor 2 1500. 10. -100. -25 Radizel 2 -1500. 10. 100.")
    self.assertFalse(self.dds._errorDict)
    assertEqualsImproved(self.dds.extraParticles, [[25, "Maribor", 2, 1500., 10., -100.],
                                                   [-25, "Radizel", 2, -1500., 10., 100.]], self)

  def test_addextraparticles_fails(self):
    """Test if API detects wrong input."""
    self.assertFalse(self.dds._errorDict)
    ret = self.dds.setExtraParticles('abc')
    self.assertIn("multiples of 6", ret['Message'])
    self.assertIn('setExtraParticles', self.dds._errorDict)

  def test_addextraparticles_fail2(self):
    """Test if API detects wrong input."""
    self.assertFalse(self.dds._errorDict)
    ret = self.dds.setExtraParticles("25 Ptuj 2 1500. 10. 100. 10.")
    self.assertIn("multiples of 6", ret['Message'])
    self.assertIn('setExtraParticles', self.dds._errorDict)

  def test_addextraparticles_fail3(self):
    """Test if API detects wrong input."""
    self.assertFalse(self.dds._errorDict)
    ret = self.dds.setExtraParticles("25.0 Sentilj 2 1500. 10. 100.")
    self.assertIn("Cannot convert input string to proper format", ret['Message'])
    self.assertIn('setExtraParticles', self.dds._errorDict)

  def test_addextraparticles_fail4(self):
    """Test if API detects wrong input."""
    self.assertFalse(self.dds._errorDict)
    ret = self.dds.setExtraParticles("25 Graz 2 a 10. 100.")
    self.assertIn("Cannot convert input string to proper format", ret['Message'])
    self.assertIn('setExtraParticles', self.dds._errorDict)

  def test_setstartfrom(self):
    self.assertFalse(self.dds._errorDict)
    self.dds.setStartFrom(89421)
    self.assertFalse(self.dds._errorDict)
    assertEqualsImproved(self.dds.startFrom, 89421, self)

  def test_setstartfrom_fails(self):
    self.assertFalse(self.dds._errorDict)
    self.dds.setStartFrom('adgiuj')
    self.assertIn('_checkArgs', self.dds._errorDict)

  def test_resolvelinkedparams(self):
    step_mock = Mock()
    input_mock = Mock()
    input_mock.getType.return_value = {'abc': False}
    self.dds._linkedidx = 3
    self.dds._jobsteps = [None, None, None, input_mock]
    assertDiracSucceeds(self.dds._resolveLinkedStepParameters(step_mock), self)
    step_mock.setLink.assert_called_once_with('InputFile', {'abc': False}, 'OutputFile')

  def test_resolvelinkedparams_noinputstep(self):
    self.dds._linkedidx = None
    self.dds._inputappstep = []
    assertDiracSucceeds(self.dds._resolveLinkedStepParameters(None), self)

  def test_checkworkflow_app_missing(self):
    self.dds._inputapp = ['some_depdency', 'unavailable_dependency_fail_on_this']
    self.dds._jobapps = ['myjobapp_1', 'some_dependency']
    assertDiracFailsWith(self.dds._checkWorkflowConsistency(), 'job order not correct', self)

  def test_checkworkflow_empty(self):
    self.dds._inputapp = []
    self.dds._jobapps = []
    assertDiracSucceeds(self.dds._checkWorkflowConsistency(), self)

  def test_checkworkflow_success(self):
    self.dds._inputapp = ['some_dependency', 'other_dependencies', 'many_more']
    self.dds._jobapps = ['ignore_me', 'many_more', 'some_dependency', 'other_dependencies']
    assertDiracSucceeds(self.dds._checkWorkflowConsistency(), self)

  def test_userjobmodules(self):
    module_mock = Mock()
    assertDiracSucceeds(self.dds._userjobmodules(module_mock), self)

  def test_prodjobmodules(self):
    module_mock = Mock()
    assertDiracSucceeds(self.dds._prodjobmodules(module_mock), self)

  def test_userjobmodules_fails(self):
    with patch('%s._setUserJobFinalization' % MODULE_NAME, new=Mock(return_value=S_OK('something'))),\
        patch('%s._setApplicationModuleAndParameters' % MODULE_NAME, new=Mock(return_value=S_ERROR('some_test_err'))):
      assertDiracFailsWith(self.dds._userjobmodules(None),
                            'userjobmodules failed', self)

  def test_prodjobmodules_fails(self):
    with patch('%s._setApplicationModuleAndParameters' % MODULE_NAME, new=Mock(return_value=S_OK('something'))), \
        patch('%s._setOutputComputeDataList' % MODULE_NAME, new=Mock(return_value=S_ERROR('some_other_test_err'))):
      assertDiracFailsWith(self.dds._prodjobmodules(None),
                            'prodjobmodules failed', self)

  def test_checkconsistency(self):
    self.dds.version = '134'
    self.dds.detectorModel = 'mymodel.det'
    self.dds.outputFile = 'myoutput.file'
    self.dds._jobtype = 'User'
    assertDiracSucceeds(self.dds._checkConsistency(Mock()), self)
    self.assertNotIn({'outputFile': '@{OutputFile}', 'outputPath': '@{OutputPath}',
                        'outputDataSE': '@{OutputSE}'}, self.dds._listofoutput)
    self.assertNotIn('nbevts', self.dds.prodparameters)
    self.assertNotIn('Process', self.dds.prodparameters)

  def test_checkconsistency_extraparticles(self):
    """Check if consistency check for extra particles passes."""
    self.dds.version = '134'
    self.dds.detectorModel = 'mymodel.det'
    self.dds.outputFile = 'myoutput.file'
    self.dds._jobtype = 'User'
    self.dds.extraParticles = [[25, "Maribor", 2, 1500., 10., 100.]]
    assertDiracSucceeds(self.dds._checkConsistency(Mock()), self)

  def test_checkconsistency_extraparticles1(self):
    """Check if consistency check for extra particles detects wrong type."""
    self.dds.version = '134'
    self.dds.detectorModel = 'mymodel.det'
    self.dds.outputFile = 'myoutput.file'
    self.dds._jobtype = 'User'
    self.dds.extraParticles = [25, "Maribor", 2, 1500., 10., 100.]
    assertDiracFailsWith(self.dds._checkConsistency(Mock()), 'Wrong format', self)

  def test_checkconsistency_extraparticles2(self):
    """Check if consistency check for extra particles detects too long input."""
    self.dds.version = '134'
    self.dds.detectorModel = 'mymodel.det'
    self.dds.outputFile = 'myoutput.file'
    self.dds._jobtype = 'User'
    self.dds.extraParticles = [[25, "Maribor", 2, 1500., 10., 100., 10]]
    assertDiracFailsWith(self.dds._checkConsistency(Mock()), 'Particle property not of correct format', self)

  def test_checkconsistency_extraparticles3(self):
    """Check if consistency check for extra particles detects first argument correctly."""
    self.dds.version = '134'
    self.dds.detectorModel = 'mymodel.det'
    self.dds.outputFile = 'myoutput.file'
    self.dds._jobtype = 'User'
    self.dds.extraParticles = [[25., "Maribor", 2, 1500., 10., 100.]]
    assertDiracFailsWith(self.dds._checkConsistency(Mock()), 'Particle property ID not int', self)

  def test_checkconsistency_extraparticles4(self):
    """Check if consistency check for extra particles detects second argument correctly."""
    self.dds.version = '134'
    self.dds.detectorModel = 'mymodel.det'
    self.dds.outputFile = 'myoutput.file'
    self.dds._jobtype = 'User'
    self.dds.extraParticles = [[25, 1, 2, 1500., 10., 100.]]
    assertDiracFailsWith(self.dds._checkConsistency(Mock()), 'Particle property name not string', self)

  def test_checkconsistency_extraparticles5(self):
    """Check if consistency check for extra particles detects third argument correctly."""
    self.dds.version = '134'
    self.dds.detectorModel = 'mymodel.det'
    self.dds.outputFile = 'myoutput.file'
    self.dds._jobtype = 'User'
    self.dds.extraParticles = [[25, "Maribor", 2., 1500., 10., 100.]]
    assertDiracFailsWith(self.dds._checkConsistency(Mock()), 'Particle property chg not int', self)

  def test_checkconsistency_extraparticles6(self):
    """Check if consistency check for extra particles detects fourth argument correctly."""
    self.dds.version = '134'
    self.dds.detectorModel = 'mymodel.det'
    self.dds.outputFile = 'myoutput.file'
    self.dds._jobtype = 'User'
    self.dds.extraParticles = [[25, "Maribor", 2, 1500, 10., 100.]]
    assertDiracFailsWith(self.dds._checkConsistency(Mock()), 'Particle property mass not float', self)

  def test_checkconsistency_extraparticles7(self):
    """Check if consistency check for extra particles detects fifth argument correctly."""
    self.dds.version = '134'
    self.dds.detectorModel = 'mymodel.det'
    self.dds.outputFile = 'myoutput.file'
    self.dds._jobtype = 'User'
    self.dds.extraParticles = [[25, "Maribor", 2, 1500., "10.", 100.]]
    assertDiracFailsWith(self.dds._checkConsistency(Mock()), 'Particle property total width not float', self)

  def test_checkconsistency_extraparticles8(self):
    """Check if consistency check for extra particles detects sixth argument correctly."""
    self.dds.version = '134'
    self.dds.detectorModel = 'mymodel.det'
    self.dds.outputFile = 'myoutput.file'
    self.dds._jobtype = 'User'
    self.dds.extraParticles = [[25, "Maribor", 2, 1500., 10., 'test']]
    assertDiracFailsWith(self.dds._checkConsistency(Mock()), 'Particle property lifetime not float', self)

  def test_checkconsistency_nodetectormodel(self):
    self.dds.version = 123
    self.dds.steeringFile = None
    self.dds.detectorModel = None
    assertDiracFailsWith(self.dds._checkConsistency(Mock()), 'no detectormodel set', self)

  def test_checkconsistency_noversion(self):
    self.dds.version = None
    assertDiracFailsWith(self.dds._checkConsistency(Mock()), 'no version found', self)

  def test_checkconsistency_existsfails(self):
    self.dds.version = '134'
    self.dds.steeringFile = 'mysteer.file'
    with patch('os.path.exists', new=Mock(return_value=False)), \
         patch.object(inspect.getmodule(DDSim), 'Exists', new=Mock(return_value=S_ERROR('testerr_exists_mock'))):
      assertDiracFailsWith(self.dds._checkConsistency(Mock()), 'testerr_exists_mock', self)

  def test_checkconsistency_userjob(self):
    self.dds.version = '134'
    self.dds.steeringFile = 'mysteer.file'
    self.dds._jobtype = 'notUser'
    self.dds.detectorModel = 'myDetectorv200'
    with patch('os.path.exists', new=Mock(return_value=True)), \
         patch.object(inspect.getmodule(DDSim), 'Exists', new=Mock(return_value=S_ERROR('testerr_exists_mock'))):
      assertDiracSucceeds(self.dds._checkConsistency(Mock()), self)
      self.assertIn({'outputFile': '@{OutputFile}', 'outputPath': '@{OutputPath}',
                       'outputDataSE': '@{OutputSE}'}, self.dds._listofoutput)
      for keyword in ['detectorType', 'slic_detectormodel']:
        self.assertIn(keyword, self.dds.prodparameters)

  def test_checkconsistency_userjob_notdetmodel(self):
    self.dds.version = '134'
    self.dds.steeringFile = 'mysteer.file'
    self.dds._jobtype = 'notUser'
    self.dds.detectorModel = True
    self.dds.setStartFrom(148)
    with patch('os.path.exists', new=Mock(return_value=False)), \
         patch.object(inspect.getmodule(DDSim), 'Exists', new=Mock(return_value=S_OK())):
      assertDiracSucceeds(self.dds._checkConsistency(Mock()), self)
      self.assertIn({'outputFile': '@{OutputFile}', 'outputPath': '@{OutputPath}',
                       'outputDataSE': '@{OutputSE}'}, self.dds._listofoutput)
      for keyword in ['detectorType', 'slic_detectormodel']:
        self.assertIn(keyword, self.dds.prodparameters)

# pylint: disable=protected-access


class TestDDSim(unittest.TestCase):
  """tests for the DDSim interface."""

  def setUp(self):
    pass

  def tearDown(self):
    """cleanup any files."""
    pass

  @patch("ILCDIRAC.Interfaces.API.NewInterface.Applications.DDSim.getKnownDetectorModels",
          new=Mock(return_value=S_OK({'CLIC_o2_v03': "/some/path"})))
  def test_setDetectorModel1(self):
    """test DDSIm setDetectorModel part of software................................................."""
    detModel = "CLIC_o2_v03"
    ddsim = DDSim()
    ddsim.setDetectorModel(detModel)
    self.assertEqual(ddsim.detectorModel, detModel)

  @patch("ILCDIRAC.Interfaces.API.NewInterface.Applications.DDSim.getKnownDetectorModels",
          new=Mock(return_value=S_ERROR("No known models")))
  def test_setDetectorModel2(self):
    """test DDSIm setDetectorModel part of software failure........................................."""
    detModel = "CLIC_o2_v03"
    ddsim = DDSim()
    res = ddsim.setDetectorModel(detModel)
    self.assertIn("No known models", res['Message'])

  @patch("ILCDIRAC.Interfaces.API.NewInterface.Applications.DDSim.getKnownDetectorModels",
          new=Mock(return_value=S_OK({'CLIC_o2_v04': "/some/path"})))
  def test_setDetectorModel3(self):
    """test DDSIm setDetectorModel is not known....................................................."""
    detModel = "ATLAS"
    ddsim = DDSim()
    ret = ddsim.setDetectorModel(detModel)
    self.assertEqual(ddsim.detectorModel, '')
    self.assertFalse(ret['OK'])
    self.assertIn("Unknown detector model in ddsim: ATLAS", ret['Message'])
    self.assertIn("setDetectorModel", ddsim._errorDict)

  @patch("os.path.exists", new=Mock(return_value=True))
  def test_setDetectorModel_TB_success(self):
    """test DDSIm setDetectorModel tarBall success.................................................."""
    detModel = "CLIC_o2_v03"
    ext = ".tar.gz"
    ddsim = DDSim()
    ddsim.setDetectorModel(detModel + ext)
    self.assertEqual(ddsim.detectorModel, detModel)
    self.assertTrue(detModel + ext in ddsim.inputSB)

  @patch("os.path.exists", new=Mock(return_value=False))
  def test_setDetectorModel_TB_notLocal(self):
    """test DDSIm setDetectorModel tarBall notLocal................................................."""
    detModel = "CLIC_o2_v03"
    ext = ".tgz"
    ddsim = DDSim()
    ddsim.setDetectorModel(detModel + ext)
    self.assertEqual(ddsim.inputSB, [])
    self.assertEqual(ddsim.detectorModel, detModel)

  def test_setDetectorModel_LFN_succcess(self):
    """test DDSIm setDetectorModel lfn success......................................................"""
    detModel = "lfn:/ilc/user/s/sailer/CLIC_o2_v03.tar.gz"
    ddsim = DDSim()
    ddsim.setDetectorModel(detModel)
    self.assertEqual(ddsim.detectorModel, "CLIC_o2_v03")
    self.assertTrue(detModel in ddsim.inputSB)

  def test_setStartFrom1(self):
    """test DDSIm setStartFrom 1...................................................................."""
    ddsim = DDSim()
    ddsim.setStartFrom("Arg")
    self.assertTrue(ddsim._errorDict)

  def test_setStartFrom2(self):
    """test DDSIm setStartFrom 2...................................................................."""
    ddsim = DDSim()
    ddsim.setStartFrom(42)
    self.assertEqual(ddsim.startFrom, 42)

  def test_getKnownDetModels1(self):
    """test getKnownDetectorModels failure no version..............................................."""
    ddsim = DDSim()
    ret = ddsim.getKnownDetectorModels()
    self.assertFalse(ret['OK'])
    self.assertEqual("No software version defined", ret['Message'])

  def test_getKnownDetModels2(self):
    """test getKnownDetectorModels success.........................................................."""
    ddsim = DDSim()
    ddsim.version = "test"
    import DIRAC
    ddsim._ops = create_autospec(DIRAC.ConfigurationSystem.Client.Helpers.Operations.Operations, spec_set=True)
    ddsim._ops.getOptionsDict.return_value = S_OK({"detModel1": "/path", "detModel2": "/path2"})
    ret = ddsim.getKnownDetectorModels()
    self.assertIn("detModel1", ret['Value'])
    self.assertTrue(ret['OK'])


def runTests():
  """Runs our tests."""
  suite = unittest.defaultTestLoader.loadTestsFromTestCase(TestDDSim)
  testResult = unittest.TextTestRunner(verbosity=2).run(suite)
  print(testResult)
  suite = unittest.defaultTestLoader.loadTestsFromTestCase(DDSimTestCase)
  testResult = unittest.TextTestRunner(verbosity=2).run(suite)
  print(testResult)


if __name__ == '__main__':
  runTests()



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Run the Post generation filter, not the same as :mod:`~ILCDIRAC.Interfaces.API.NewInterface.Applications.StdhepCut`

:since: Dec 20, 2010

:author: sposs
"""

from __future__ import absolute_import
from DIRAC.Core.Utilities.Subprocess import shellCall
from ILCDIRAC.Workflow.Modules.ModuleBase import ModuleBase
from DIRAC import S_OK, S_ERROR, gLogger
from ILCDIRAC.Core.Utilities.CombinedSoftwareInstallation import getSoftwareFolder
from ILCDIRAC.Core.Utilities.resolvePathsAndNames import getProdFilename

import os

__RCSID__ = '$Id$'
LOG = gLogger.getSubLogger(__name__)


class PostGenSelection(ModuleBase):
  """Apply cuts after generator (whizard).

  Used only by JJ Blaising up to now.
  """

  def __init__(self):
    super(PostGenSelection, self).__init__()
    self.STEP_NUMBER = ''
    self.enable = True
    self.applicationName = 'PostGenSel'
    self.InputFile = []
    self.NbEvtsKept = 0
    self.stdhepFile = ''

  def applicationSpecificInputs(self):
    """Called from ModuleBase."""
    if 'NbEvtsKept' in self.step_commons:
      self.NbEvtsKept = self.step_commons['NbEvtsKept']

    if not self.NbEvtsKept:
      return S_ERROR('Nb of events to keep MUST be specified')

    if 'IS_PROD' in self.workflow_commons:
      if self.workflow_commons["IS_PROD"]:
        # self.OutputFile = getProdFilename(self.OutputFile,int(self.workflow_commons["PRODUCTION_ID"]),
        #                                  int(self.workflow_commons["JOB_ID"]))
        if 'ProductionOutputData' in self.workflow_commons:
          outputlist = self.workflow_commons['ProductionOutputData'].split(";")
          for obj in outputlist:
            if obj.lower().count("_gen_"):
              self.InputFile = [os.path.basename(obj)]
              self.OutputFile = self.InputFile[0]
        else:
          if 'WhizardOutput' in self.workflow_commons:
            self.stdhepFile = getProdFilename(self.workflow_commons["WhizardOutput"],
                                              int(self.workflow_commons["PRODUCTION_ID"]),
                                              int(self.workflow_commons["JOB_ID"]))

      if not len(self.InputFile) and len(self.InputData):
        for files in self.InputData:
          if files.lower().count(".stdhep"):
            self.InputFile.append(files)
            break

    return S_OK('Parameters resolved')

  def execute(self):
    """Run it now."""
    self.result = self.resolveInputVariables()
    if not self.applicationLog:
      self.result = S_ERROR('No Log file provided')
    if not self.platform:
      self.result = S_ERROR('No ILC platform selected')
    if not self.result['OK']:
      LOG.error("Failed to resolve input parameters:", self.result["Message"])
      return self.result

    if not self.workflowStatus['OK'] or not self.stepStatus['OK']:
      LOG.verbose('Workflow status = %s, step status = %s' % (self.workflowStatus['OK'], self.stepStatus['OK']))
      return S_OK('PostGenSelection should not proceed as previous step did not end properly')

    if 'ROOTSYS' not in os.environ:
      return S_OK('Root environment is not set')
    res = getSoftwareFolder(self.platform, "postgensel", self.applicationVersion)
    if not res['OK']:
      LOG.error("Failed finding the sofware")
      self.setApplicationStatus('PostGenSel: Could not find neither local area not shared area install')
      return res

    mySoftDir = res['Value']
    InputFile = os.path.basename(self.InputFile[0])
    base_file = InputFile.replace(".stdhep", "")

    scriptName = 'PostGenSel_%s_Run_%s.sh' % (self.applicationVersion, self.STEP_NUMBER)
    if os.path.exists(scriptName):
      os.remove(scriptName)
    script = open(scriptName, 'w')
    script.write('#!/bin/sh \n')
    script.write('#####################################################################\n')
    script.write('# Dynamically generated script to run a production or analysis job. #\n')
    script.write('#####################################################################\n')
    script.write('declare -x PATH=%s:$PATH\n' % mySoftDir)
    script.write('env | sort >> localEnv.log\n')
    script.write('echo =============================\n')
    script.write('declare -x DEBUG=ON\n')
    script.write('declare -x INDIR=$PWD/\n')
    script.write('declare -x MCGEN=WHIZARD\n')
    comm = "readstdhep 100000 %s\n" % base_file
    LOG.info("Running %s" % comm)
    script.write(comm)
    script.write('declare -x appstatus=$?\n')
    script.write('exit $appstatus\n')
    script.close()
    os.chmod(scriptName, 0o755)
    comm = 'sh -c "./%s"' % (scriptName)
    self.setApplicationStatus('PostGenSelection_Read %s step %s' % (self.applicationVersion, self.STEP_NUMBER))
    self.stdError = ''
    self.result = shellCall(0, comm, callbackFunction=self.redirectLogOutput, bufferLimit=20971520)
    resultTuple = self.result['Value']
    status = resultTuple[0]
    if not status == 0:
      LOG.error("Reading did not proceed properly")
      self.setApplicationStatus('PostGenSelection_Read Exited With Status %s' % (status))
      return S_ERROR('PostGenSelection_Read Exited With Status %s' % (status))

    if not os.path.exists(base_file + ".dat"):
      LOG.error('%s does not exist locally, something went wrong, cannot proceed' % (base_file + ".dat"))
      self.setApplicationStatus('%s not there!' % (base_file + ".dat"))
      return S_ERROR('%s file does not exist' % (base_file + ".dat"))

    os.rename(base_file + ".stdhep", base_file + "-old.stdhep")

    os.remove(scriptName)
    script = open(scriptName, 'w')
    script.write('#!/bin/sh \n')
    script.write('#####################################################################\n')
    script.write('# Dynamically generated script to run a production or analysis job. #\n')
    script.write('#####################################################################\n')
    script.write('declare -x PATH=%s:$PATH\n' % mySoftDir)
    script.write('env | sort >> localEnv.log\n')
    script.write('echo =============================\n')
    script.write('declare -x DEBUG=ON\n')
    script.write('declare -x INDIR=$PWD/\n')
    script.write('declare -x MCGEN=WHIZARD\n')
    comm = 'writestdhep 100000 %s %s > writestdhep.out\n' % (self.NbEvtsKept, base_file)
    LOG.info('Running %s' % comm)
    script.write(comm)
    script.write('declare -x appstatus=$?\n')
    script.write('exit $appstatus\n')
    script.close()
    os.chmod(scriptName, 0o755)
    comm = 'sh -c "./%s"' % (scriptName)
    self.setApplicationStatus('PostGenSelection_Write %s step %s' % (self.applicationVersion, self.STEP_NUMBER))
    self.stdError = ''
    self.result = shellCall(0, comm, callbackFunction=self.redirectLogOutput, bufferLimit=20971520)
    resultTuple = self.result['Value']
    status = resultTuple[0]

    # need to update the number of events kept
    self.workflow_commons["NbOfEvts"] = self.NbEvtsKept

    return self.finalStatusReport(status)



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""ILD DBD specific  job utility.

:author: S. Poss, A. Sailer, C. Calancha
:since: Jul 01, 2012
"""

from __future__ import print_function
from __future__ import absolute_import
import string
import pprint
from decimal import Decimal


from DIRAC.Core.Workflow.Module import ModuleDefinition
from DIRAC.Core.Workflow.Step import StepDefinition
from DIRAC import S_OK, S_ERROR

from ILCDIRAC.Interfaces.API.NewInterface.ProductionJob import ProductionJob
from ILCDIRAC.Interfaces.Utilities import JobHelpers
from ILCDIRAC.Core.Utilities.LFNPathUtilities import joinPathForMetaData
import six

__RCSID__ = "$Id$"

# pylint: disable=W0311
# pylint: disable=R0902
# pylint: disable=R0904


class ILDProductionJobDBD(ProductionJob):
  """ILD Production Jobs definition."""

  def __init__(self):
    super(ILDProductionJobDBD, self).__init__()
    self.machine = 'ilc'
    self.experiment = 'ILC_ILD'
    self.basepath = self.ops.getValue('/Production/%s/BasePath' % self.experiment, '/ilc/prod/ilc/mc-dbd/ild/')
    self.polarization = ""
    self.machineparams = ''
    self.detector = ''
    self.compatmeta = {}
    self.processID = 0
    self.evtclass = ''
    self.evttype = ''
    self.genprocname = ''
    self.matchToInput = ''

  def setMatchToInput(self, matchToInput):
    """Help to find faster the input directory."""
    self.matchToInput = matchToInput

  def setProcessIDInFinalPath(self):
    """ILD name convention dont include ProcessID in the path name ProcessID is needed in the metadata: we add in the last path."""
    for finalpaths in self.finalpaths:
      if self.processID:
        self.finalMetaDict[finalpaths].update({"ProcessID": self.processID})

    return S_OK()

  def setEvtClass(self, evtclass):
    """Sometimes we used different evtclass in the sim/reco files than the original in the stdhep."""
    self.evtclass = evtclass

  def setEvtType(self, evttype):
    """EvtType missing in input lfn: i have no privileges to set metadata on those files."""
    self.evttype = evttype
    print('[debug tino] Set self.evttype to %s ' % self.evttype)

  def setGenProcName(self, genprocname):
    """ILD convention add gen process name in the basename of LFN's."""
    self.genprocname = genprocname

  # def __swapGenProcNameEvtType( self ):
  #     """ ILD has swapped these two meta fields at different energies
  #         GenProcessName @ 350 <----> EvtType @ 1000
  #     """
  #     tmpval           = self.evttype
  #     self.evttype     = self.genprocname
  #     self.genprocname = tmpval

  def setILDConfig(self, version):
    """This is because in the ProductionJob, it's called Config."""
    return self.setConfig(version)

  def setInputDataQuery(self, metadata):
    """Define the input data query needed, also get from the data the meta info requested to build the path."""

    for key, val in metadata.items():
      print("[0] meta[%s] %s" % (key, val))

    retMetaKey = self._checkMetaKeys(list(metadata.keys()), extendFileMeta=True)
    if not retMetaKey['OK']:
      return retMetaKey

    # do i need this?
    tmp_metadata = {}
    tmp_metadata.update(metadata)
    # for kk in ['SoftwareTag', 'ILDConfig']:
    for kk in ['SoftwareTag', 'ILDConfig', 'ProcessID']:
      tmp_metadata.pop(kk, None)  # why i was dropping ProdID?

    retDirs = self._checkFindDirectories(tmp_metadata)
    if not retDirs['OK']:
      return retDirs
    dirs = list(retDirs['Value'].values())

    compatmeta = {}

    print('dirs found: %d' % len(dirs))
    # for d in dirs:
    #     print '%s'%d
    dir_found = False
    if self.matchToInput:
      print('Will try to match dir with %s' % self.matchToInput)
    for mdir in dirs:
      if self.matchToInput:
        if self.matchToInput not in mdir:
          continue
      if 'ProdID' in metadata:
        val = '/' + str(metadata['ProdID']).zfill(8)
        if val not in mdir:
          continue

      dir_found = True
      print('[debug tino] Found mdir %s' % mdir)
      res = self.fc.getDirectoryUserMetadata(mdir)
      if not res['OK']:
        return self._reportError("Error looking up the catalog for directory metadata: %s" % res['Message'])
      compatmeta = res['Value']  # this reset compatmeta for each iteration (do we want this?)
      compatmeta.update(metadata)
      print('[tino debug] Updated compatmeta to: %s' % compatmeta)

    if not dir_found:
      if self.dryrun:
        print('We could not find our target dir: please try w/o dryrun (maybe target dir still not registered)')
      else:
        print('We could not find our target dir and this is not a dryrun: please check')

    print('self.fc.findFilesByMetadata( %s, "/ilc/prod/ilc" ) ' % metadata)
    # get all the files available, if any
    res = self.fc.findFilesByMetadata(metadata, '/ilc/prod/ilc')
    # res = self.fc.findFilesByMetadata( metadata, '/ilc/user/c/calanchac/stdhep' )
    if not res['OK']:
      return self._reportError("Could not find the files with this metadata")

    if len(res['Value']):
      my_lfn = res['Value'][0]
      # #Get the meta data of the first one as it should be enough is the registration was
      # # done right

      res = self.fc.getFileUserMetadata(my_lfn)
      if not res['OK']:
        return self._reportError('Failed to get file metadata, cannot build filename: %s' % res['Message'])
      compatmeta.update(res['Value'])
      print('[tino debug] Updated compatmeta to: %s' % compatmeta)

      for key, val in self.compatmeta.items():
        print("my_lfn %s compatmeta[%s] %s" % (my_lfn, key, val))

    else:
      if not self.dryrun:
        print(res)
        self._reportError("No files matching the metadata: Metadata is wrong or files are not under "
                           "/ilc/prod/ilc directory")

    if not len(compatmeta):
      print('ERROR, compatmeta is empty: this is expected when dryrun = True')

    print('compatmeta contains ProcessID? (%s) See below:' % compatmeta)
    pprint.pprint(self.compatmeta)

    self.log.verbose("Using %s to build path" % str(compatmeta))
    if 'EvtClass' in compatmeta and not self.evtclass:
      self.evtclass = JobHelpers.getValue(compatmeta['EvtClass'], str, six.string_types)

    if 'EvtType' in compatmeta and not self.evttype:
      self.evttype = JobHelpers.getValue(compatmeta['EvtType'], str, six.string_types)
    elif 'GenProcessType' in compatmeta and not self.evttype:
      self.evttype = JobHelpers.getValue(compatmeta['GenProcessType'], str, six.string_types)
    elif not self.evttype:
      return self._reportError("Neither EvtType nor GenProcessType are "
                                "in the metadata: if you dont set app "
                                "evttype with setEvtType at least one "
                                "should be.")

    if 'GenProcessName' in compatmeta:
      self.genprocname = compatmeta['GenProcessName']

    if not self.genprocname:
      return self._reportError("GenProcessName is missing! It should appear in the basename")

    if 'GenProcessID' in compatmeta:
      self.processID = JobHelpers.getValue(compatmeta['GenProcessID'], int, six.integer_types)
    elif 'ProcessID' in compatmeta:
      self.processID = JobHelpers.getValue(compatmeta['ProcessID'], int, six.integer_types)
    else:
      return self._reportError("Cannot find ProcessID, it's mandatory for path definition")

    if 'Energy' in compatmeta:
      self.energycat = JobHelpers.getValue(compatmeta['Energy'], str, (int, int, six.string_types))

    if 'MachineParams' in compatmeta:
      self.machineparams = JobHelpers.getValue(compatmeta['MachineParams'], str, six.string_types)
    if not self.machineparams:
      return self._reportError("MachineParams should part of the metadata")
    gendata = False
    if 'Datatype' in compatmeta:
      self.datatype = JobHelpers.getValue(compatmeta['Datatype'], str, six.string_types)
      if self.datatype.lower() == 'gen':
        gendata = True

    if 'DetectorModel' in compatmeta and not gendata:
      self.detector = JobHelpers.getValue(compatmeta['DetectorModel'], str, six.string_types)

    self.compatmeta = compatmeta
    self.basename = ''
#
    if not self.energycat:  # FIXME
      print("Printing metadata before exit:")
      pprint.pprint(self.compatmeta)
      return self._reportError("ERROR::ILDProductionJobDBD.py: self.energycat is null")

    self.energy = Decimal(self.energycat)

    self.inputBKSelection = metadata
    self.prodparameters["FCInputQuery"] = self.inputBKSelection

    self.inputdataquery = True
    return S_OK()

  def _addRealFinalization(self):
    """See :mod:`~ILCDIRAC.Interfaces.API.NewInterface.ProductionJob` for definition."""
    importLine = 'from ILCDIRAC.Workflow.Modules.<MODULE> import <MODULE>'
    dataUpload = ModuleDefinition('UploadOutputData')
    dataUpload.setDescription('Uploads the output data')
    self._addParameter(dataUpload, 'enable', 'bool', False, 'EnableFlag')
    body = importLine.replace('<MODULE>', 'UploadOutputData')
    dataUpload.setBody(body)

    failoverRequest = ModuleDefinition('FailoverRequest')
    failoverRequest.setDescription('Sends any failover requests')
    self._addParameter(failoverRequest, 'enable', 'bool', False, 'EnableFlag')
    body = importLine.replace('<MODULE>', 'FailoverRequest')
    failoverRequest.setBody(body)

    registerdata = ModuleDefinition('ILDRegisterOutputData')
    registerdata.setDescription('Module to add in the metadata catalog the relevant info about the files')
    self._addParameter(registerdata, 'enable', 'bool', False, 'EnableFlag')
    body = importLine.replace('<MODULE>', 'ILDRegisterOutputData')
    registerdata.setBody(body)

    logUpload = ModuleDefinition('UploadLogFile')
    logUpload.setDescription('Uploads the output log files')
    self._addParameter(logUpload, 'enable', 'bool', False, 'EnableFlag')
    body = importLine.replace('<MODULE>', 'UploadLogFile')
    logUpload.setBody(body)

    errorReport = ModuleDefinition('ReportErrors')
    errorReport.setDescription('Reports errors at the end')
    body = importLine.replace('<MODULE>', 'ReportErrors')
    errorReport.setBody(body)

    finalization = StepDefinition('Job_Finalization')
    finalization.addModule(dataUpload)
    up = finalization.createModuleInstance('UploadOutputData', 'dataUpload')
    up.setValue("enable", self.finalsdict['uploadData'])

    finalization.addModule(registerdata)
    # TODO: create ILDRegisterOutputData
    ro = finalization.createModuleInstance('ILDRegisterOutputData', 'ILDRegisterOutputData')
    ro.setValue("enable", self.finalsdict['registerData'])

    finalization.addModule(logUpload)
    ul = finalization.createModuleInstance('UploadLogFile', 'logUpload')
    ul.setValue("enable", self.finalsdict['uploadLog'])

    finalization.addModule(failoverRequest)
    fr = finalization.createModuleInstance('FailoverRequest', 'failoverRequest')
    fr.setValue("enable", self.finalsdict['sendFailover'])

    finalization.addModule(errorReport)
    fr = finalization.createModuleInstance('ReportErrors', 'reportErrors')

    self.workflow.addStep(finalization)
    self.workflow.createStepInstance('Job_Finalization', 'finalization')

    return S_OK()

  def _jobSpecificParams(self, application):
    """For production additional checks are needed: ask the user."""
    retCheck = self.__checkParameters(application)
    if not retCheck['OK']:
      return retCheck

    if retCheck.get('Value') is not None and "ShortCut" in retCheck.get('Value'):
      # Short cut the function because of overlay
      return S_OK()

    if self.energy:  # APS: Is it possible for there no being any energy?
      self._setParameter("Energy", "float", float(self.energy), "Energy used")
      self.prodparameters["Energy"] = float(self.energy)

    energypath = "%s-%s" % (self.energy, self.machineparams)  # e.g.: 1000-B1s_ws

    retFileName = self.__createFileName(application)
    if not retFileName['OK']:
      self.log.error("Filename creation Failed", retFileName['Message'])
      return retFileName

    # Always use ILDConfig for the path
    if 'ILDConfigVersion' not in self.prodparameters and application.datatype.lower() != "gen":
      return S_ERROR("ILDConfig not set, it is mandatory for the path "
                      "definition, please use p.setILDConfig() before"
                      "appending applications")
    ildConfigPath = self.prodparameters.get("ILDConfigVersion", "") + "/"

    path = self.basepath

    if not self._recBasePaths:
      self.setReconstructionBasePaths(self.basepath, self.basepath)

    # ##Need to resolve file names and paths
    # TODO: change basepath for ILD Don't forget LOG PATH in ProductionOutpuData module
    if hasattr(application, "setOutputRecFile") and not application.willBeCut:

      for outType in ('REC', 'DST'):
        metaPath = joinPathForMetaData(self._recBasePaths[outType], outType.lower())
        self.finalMetaDict[metaPath] = {'Datatype': outType}

        metaPath = joinPathForMetaData(metaPath, energypath)
        self.finalMetaDict[metaPath] = {'Energy': str(self.energy),
                                           'MachineParams': self.machineparams}

        metaPath = joinPathForMetaData(metaPath, self.evttype)
        self.finalMetaDict[metaPath] = {'EvtType': self.evttype.strip('/')}

        metaPath = joinPathForMetaData(metaPath, self.detector)
        self.finalMetaDict[metaPath] = {'DetectorModel': self.detector.strip('/')}

        metaPath = joinPathForMetaData(metaPath, ildConfigPath)
        self.finalMetaDict[metaPath] = {'ILDConfig': self.prodparameters['ILDConfigVersion']}

        fname = "%s_%s.slcio" % (self.basename, outType.lower())
        print('+++Output %s Filename: %s' % (outType, fname))
        getattr(application, 'setOutput%sFile' % outType.capitalize())(fname, metaPath)
        self.finalpaths.append(metaPath)
        path = metaPath

    elif hasattr(application, 'outputFile') and \
         hasattr(application, 'datatype') and \
         not application.outputFile and \
         not application.willBeCut:

      if not application.datatype and self.datatype:
        application.datatype = self.datatype

      path_gen_or_sim = ''
      if application.datatype == 'gen':
        # for historical reasons generated stdhep separated from mokka/dst/rec files

        # Set DataType
        path_gen_or_sim = joinPathForMetaData('/'.join(self.basepath.split('/')[:-2]) + '/splitted/')
        self.finalMetaDict[path_gen_or_sim] = {'Datatype': 'gen'}

        # Set MachineParams and Energy
        path_gen_or_sim = joinPathForMetaData(path_gen_or_sim, energypath)
        self.finalMetaDict[path_gen_or_sim] = {'Energy': str(self.energy),
                                                  'MachineParams': self.machineparams}

        path_gen_or_sim = joinPathForMetaData(path_gen_or_sim, self.evttype)
        self.finalMetaDict[path_gen_or_sim] = {'EvtType': self.evttype.strip('/')}

      elif application.datatype == 'SIM':
        # Set DataType
        path_gen_or_sim = joinPathForMetaData(self.basepath + 'sim')
        self.finalMetaDict[path_gen_or_sim] = {'Datatype': 'SIM'}

        # Set Energy and MachineParams
        path_gen_or_sim = joinPathForMetaData(path_gen_or_sim, energypath)
        self.finalMetaDict[path_gen_or_sim] = {'Energy': str(self.energy),
                                                  'MachineParams': self.machineparams}

        # Set EventType
        path_gen_or_sim = joinPathForMetaData(path_gen_or_sim, self.evttype)
        self.finalMetaDict[path_gen_or_sim] = {'EvtType': self.evttype.strip('/')}

        # Set DetectorModel
        path_gen_or_sim = joinPathForMetaData(path_gen_or_sim, self.detector)
        self.finalMetaDict[path_gen_or_sim] = {'DetectorModel': self.detector.strip('/')}

        # Always use ILDConfig
        path_gen_or_sim = joinPathForMetaData(path_gen_or_sim, ildConfigPath)
        self.finalMetaDict[path_gen_or_sim].update({'ILDConfig': self.prodparameters['ILDConfigVersion']})

      path = path_gen_or_sim

      path_gen_or_sim = path_gen_or_sim.rstrip('/') + '/'

      self.log.info('Will store the files under: %s' % path_gen_or_sim)
      self.finalpaths.append(path_gen_or_sim)

      extension = 'stdhep'
      if application.datatype in ['SIM', 'REC']:
        extension = 'slcio'
      fname = self.basename + "_%s" % (application.datatype.lower()) + "." + extension
      application.setOutputFile(fname, path_gen_or_sim)
      print("+++Output SIM/GEN Filename", fname)

    # Applied for all productions, these are the metadata at the production level
    metap = {}
    # Drop ILDConfig, MachineParams, Energy from this list, they are set at different level
    # SoftwareTag is only sometimes at the prodID level
    for imeta in ['GenProcessName',
                  'NumberOfEvents',
                  'BeamParticle1', 'BeamParticle2',
                  'PolarizationB1', 'PolarizationB2']:
      if imeta in self.compatmeta:
        print('Updating final metadata with {"%s":"%s"}' % (imeta, self.compatmeta[imeta]))
        metap.update({imeta: self.compatmeta[imeta]})

    # Add software to non-searchable metadata
    curpackage = "%s.%s" % (application.appname, application.version)
    if "SWPackages" in self.prodparameters:
      if not self.prodparameters["SWPackages"].count(curpackage):
        self.prodparameters["SWPackages"] += ";%s" % curpackage
    else:
      self.prodparameters["SWPackages"] = curpackage

    softmeta = application.appname + "." + application.version
    print("++++Software meta for", application.appname, "=", softmeta)
    metap.update({"SoftwareTag": softmeta})

    self.prodMetaDict.update(metap)
    pprint.pprint(self.prodMetaDict)
    self.basepath = path

    self.checked = True

    return S_OK()

  def append(self, application):
    """Append application to production job, in addition to `Job.append` calls checkProductionMetadata for applications.

    :param application: Application instance
    :type application: :mod:`~ILCDIRAC.Interfaces.API.NewInterface.Application`
    :returns: S_OK, S_ERROR
    """

    # First check meta data before doing the normal append
    resMD = application.checkProductionMetaData(self.compatmeta)
    if not resMD['OK']:
      self.log.error("Failed to check production Meta Data", resMD['Message'])
      return resMD

    return super(ILDProductionJobDBD, self).append(application)

  def __createFileName(self, application):  # pylint: disable=too-many-branches
    """create the filename for ILD productions.

    A partial description of the desired filename is found here
    https://svnweb.cern.ch/trac/lcgentools/browser/trunk/ILC/documents/generator-conventions.pdf

    :param application: application used for production job
    :returns: S_OK, S_ERROR
    """
    # TODO: Make sure basename is correct. Maybe allow for setting basename prefix
    # Final name being e.g. NAME_rec.slcio, need to define NAME, maybe based on meta data (include
    # EvtClass automatically)
    if not self.basename:
      if 'ILDConfigVersion' in self.prodparameters:
        if application.appname in ('mokka', 'ddsim'):  # sim
          self.basename = 's' + self.prodparameters['ILDConfigVersion']
        elif application.appname == 'marlin':  # reco
          self.basename = 'r' + self.prodparameters['ILDConfigVersion']
          self.basename += '.s' + self.compatmeta['ILDConfig']
        # we dont need this tag in stdhep's: metadata search will fail
        # if not present
        elif application.appname == 'stdhepsplit':
          self.compatmeta.pop('SoftwareTag', None)
          self._reportError("Drop 'SoftwareTag' from metadata: not needed for stdhepsplit app")
        # need extension if planning to use additional modules (LCIOSplit)
      else:
        if application.datatype not in ('gen', 'gensplit'):  # for stdhepsplit we dont need to return
          self._reportError(" Printing metadata before exit:")
          pprint.pprint(self.compatmeta)
          pprint.pprint(self.prodparameters)
          return self._reportError("'ILDConfigVersion' should be defined to build the path")

    if 'DetectorModel' in self.compatmeta:
      self.basename += '.m' + self.compatmeta['DetectorModel']
    elif self.detector:
      self.basename += '.m' + self.detector

    if self.energy:
      self.basename += '.' if self.basename else ''
      self.basename += 'E' + str(self.energy)

      if 'MachineParams' in self.compatmeta:
        self.basename += '-' + self.compatmeta['MachineParams']

    if 'GenProcessID' in self.compatmeta:
      self.basename += '.I' + str(self.compatmeta['GenProcessID'])
    elif 'ProcessID' in self.compatmeta:
      self.basename += '.I' + str(self.compatmeta['ProcessID'])

    if 'GenProcessName' in self.compatmeta:
      self.basename += '.P' + self.compatmeta['GenProcessName']
    elif self.genprocname:
      self.basename += '.P' + self.genprocname
    else:
      return self._reportError("GenProcessName is missing! It should appear in the basename")

    # always use e and p for beam polarisation fields
    self.basename += '.e%s' % self.compatmeta.get('PolarizationB1', '')
    self.basename += '.p%s' % self.compatmeta.get('PolarizationB2', '')

    return S_OK()

  def __checkParameters(self, application):  # pylint: disable=too-many-return-statements, too-many-branches
    """check if everything is consistent, parameters set, meta data defined, applications configured correctly...

    :returns: S_OK, S_ERROR
    """

    if self.created:
      return S_ERROR("The production was created, you cannot add new applications to the job.")

    curpackage = "%s.%s" % (application.appname, application.version)
    self.prodparameters.setdefault('SoftwareTag', curpackage)
    if curpackage not in self.prodparameters["SoftwareTag"]:
      self.prodparameters["SoftwareTag"] += ";%s" % curpackage

    if not application.logFile:
      logf = application.appname + "_" + application.version + "_@{STEP_ID}.log"
      resLog = application.setLogFile(logf)
      if not resLog['OK']:
        return resLog

    if not self.energy:
      if application.energy:
        self.energy = Decimal(str(application.energy))
      else:
        return S_ERROR("Could not find the energy defined, it is needed for the production definition.")
    elif not application.energy:
      resEnergy = application.setEnergy(float(self.energy))
      if not resEnergy['OK']:
        return resEnergy

    if not self.evttype:
      if hasattr(application, 'evttype'):
        self.evttype = application.evtType
      else:
        return S_ERROR("Event type not found nor specified, it is mandatory for the production paths.")

    if not application.accountInProduction:
      # needed for the overlay
      resUPP = self._updateProdParameters(application)
      if not resUPP['OK']:
        return resUPP
      self.checked = True
      return S_OK("ShortCut")

    if not self.outputStorage:
      return S_ERROR("You need to specify the Output storage element")

    resSE = application.setOutputSE(self.outputStorage)
    if not resSE['OK']:
      return resSE

    if not self.detector:
      if hasattr(application, "detectorModel"):
        self.detector = application.detectorModel
        if not self.detector:
          return S_ERROR("Application does not know which model to use, so the production does not either.")

    return S_OK()



#!/usr/bin/env python
#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Test user jobfinalization."""
from __future__ import print_function
from __future__ import absolute_import
__RCSID__ = "$Id$"

import unittest
import os
import shutil
import six.moves.urllib.request
import six.moves.urllib.error
import six.moves.urllib.parse
import tempfile
import ssl

from DIRAC import gLogger
#from ILCDIRAC.Workflow.Modules.SLICPandoraAnalysis import SLICPandoraAnalysis

gLogger.setLevel("ERROR")
gLogger.showHeaders(True)


def cleanup(tempdir):
  """Remove files after run."""
  try:
    shutil.rmtree(tempdir)
  except OSError:
    pass


class TestSlicPandoraAnalysis(unittest.TestCase):
  """test SlicPandoraAnalysis."""

  def setUp(self):
    self.curdir = os.getcwd()
    self.tempdir = tempfile.mkdtemp("", dir="./")
    os.chdir(self.tempdir)

    super(TestSlicPandoraAnalysis, self).setUp()
    #self.spa = SLICPandoraAnalysis()
    self.mydir = "temp_dir"
    #detURL = "http://www.lcsim.org/detectors/clic_sid_cdr.zip"
    detURL = "https://lcd-data.web.cern.ch/lcd-data/ILCDIRACTars/testfiles/clic_sid_cdr.zip"
    self.zipfile = "clic_sid_cdr.zip"

    attempts = 0

    # create a fake ssl context to avoid checking certificate
    ctx = ssl.create_default_context()
    ctx.check_hostname = False
    ctx.verify_mode = ssl.CERT_NONE

    while attempts < 3:
      try:
        response = six.moves.urllib.request.urlopen(detURL, timeout=5, context=ctx)
        content = response.read()
        with open(self.zipfile, 'wb') as zipF:
          zipF.write(content)
        break
      except six.moves.urllib.error.URLError as e:
        attempts += 1
        print(type(e), str(e))

  def tearDown(self):
    os.chdir(self.curdir)
    cleanup(self.tempdir)

  def test_Unzip_file_into_dir(self):
    """test unzip_file_into_dir....................................................................."""

    from ILCDIRAC.Core.Utilities.CombinedSoftwareInstallation import unzip_file_into_dir
    if not os.path.exists(self.mydir):
      os.mkdir(self.mydir)
    unzip_file_into_dir(self.zipfile, self.mydir)
    unzip_file_into_dir(self.zipfile, self.mydir)
    self.assertTrue(os.path.exists(os.path.join(self.mydir, "compact.xml")))


def runTests():
  """Runs our tests."""
  suite = unittest.defaultTestLoader.loadTestsFromTestCase(TestSlicPandoraAnalysis)
  testResult = unittest.TextTestRunner(verbosity=2).run(suite)
  print(testResult)


if __name__ == '__main__':
  runTests()



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Goes with DBDGeneration, not used.

:since: Jan 26, 2012

:author: Stephane Poss
"""

from __future__ import absolute_import
from ILCDIRAC.Workflow.Modules.ModuleBase import ModuleBase

from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient

from DIRAC import S_OK, gLogger

__RCSID__ = '$Id$'
LOG = gLogger.getSubLogger(__name__)


class DBDGenRegisterOutputData(ModuleBase):
  """Normally, was supposed to be used to produce the DBD gen level files.

  Dropped in the end.
  """

  def __init__(self):
    super(DBDGenRegisterOutputData).__init__()
    self.version = "DBDGenRegisterOutputData v1"
    self.commandTimeOut = 10 * 60
    self.enable = True
    self.fcc = FileCatalogClient()
    self.nbofevents = 0
    self.prodOutputLFNs = []

  def applicationSpecificInputs(self):
    if 'ProductionOutputData' in self.workflow_commons:
      self.prodOutputLFNs = self.workflow_commons['ProductionOutputData'].split(";")

    if 'NbOfEvts' in self.workflow_commons:
      self.nbofevents = self.workflow_commons['NbOfEvts']
    return S_OK("Parameters resolved")

  def execute(self):
    LOG.info('Initializing %s' % self.version)
    result = self.resolveInputVariables()
    if not result['OK']:
      LOG.error(result['Message'])
      return result

    if not self.workflowStatus['OK'] or not self.stepStatus['OK']:
      LOG.verbose('Workflow status = %s, step status = %s' % (self.workflowStatus['OK'], self.stepStatus['OK']))
      return S_OK('No registration of output data metadata attempted')

    if not len(self.prodOutputLFNs):
      LOG.info('No production data found, so no metadata registration to be done')
      return S_OK("No files' metadata to be registered")

    LOG.verbose("Will try to set the metadata for the following files: \n %s" % '\n'.join(self.prodOutputLFNs))

    for files in self.prodOutputLFNs:
      metadict = {}
      metadict['NumberOfEvents'] = self.nbofevents
      path = files

      res = self.fcc.setMetadata(files, metadict)
      if not res['OK']:
        LOG.error("Could not register %s for %s" % (metadict, path))
        return res

    return S_OK()



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Python script to generate 10x100 events for each of the required particles."""
# !/usr/bin/env python

from __future__ import print_function

from __future__ import absolute_import
from six.moves import range
ILCSOFT_DIR = ''


def main():
  """Run the generation process.

  :returns: exit status. 0 on success, 1 on failure
  :rtype: int
  """
  # Form: (particle_name, energy_in_gev), both strings
  particles = [('gamma', '10'), ('mu-', '10'), ('kaon0L', '50')]
  number_of_iterations = 10  # 10 will generate 1000 = 10 x 100 events per particle
  for (particle_name, particle_energy) in particles:
    for i in range(0, number_of_iterations):
      run_script(particle_name, particle_energy, i)
  return 0


def run_script(particle_name, particle_energy, index):
  """Run the script using the passed parameters.

  :param str particle_name: Name of the particle
  :param str particle_energy: Energy of the particle
  :param int index: index of the inner loop, used to index the file
  :returns: None
  :rtype: None
  """
  import subprocess
  res = subprocess.check_output(
      ['ddsim', '--steeringFile', '%s/ClicPerformance/HEAD/examples/clic_steer.py' % ILCSOFT_DIR,
       '--compactFile', ('/cvmfs/clicdp.cern.ch/iLCSoft/builds/2017-02-17/x86_64-slc6-gcc62-opt/lcgeo/HEAD/CLIC/compact'
                         '/CLIC_o3_v08/CLIC_o3_v08.xml'),
       '--enableGun', '--gun.particle', particle_name, '--gun.energy',
       '%s*GeV' % particle_energy, '--gun.distribution', 'uniform',
       '--outputFile', ('/afs/cern.ch/user/j/jebbing/particles/CLIC_o3_v08/%s/%s/event_%d.slcio'
                        % (particle_name, particle_energy, index)),
       '--numberOfEvents', '100'])
  print(res)


if __name__ == '__main__':
  import os
  if 'ILCSOFT' not in os.environ:
    print('Environment variable ILCSOFT not set, please run init_ilcsoft.sh')
    exit(1)
  ILCSOFT_DIR = os.environ['ILCSOFT']

  exit(main())
